/*BEGIN CMVC */
//*************************************************************************
//
//  Workfile: servlet/sample/AuthenticationHandler.java, v3-java, fmv3
//  Last update: 01/05/09 16:14:50
//  SCCS path, id: /home/flowmark/vc/0/3/6/1/s.43 1.10
//
//************************************************************************/
/*END CMVC */
/*BEGIN COPYRIGHT */
//*****************************************************************************
//
//  Licensed Materials - Property of IBM
//  5697-FM3 5655-A96
//  (C) Copyright IBM Corp. 1999, 2008
//  US Government Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract
//  with IBM Corp.
//
//****************************************************************************/
//END COPYRIGHT */
/*BEGIN DISCLAIMER */
//*************************************************************************
//  DISCLAIMER
//  ----------
//  This material contains programming source code for your consideration.
//  These examples have not been thoroughly tested under all conditions.
//  IBM, therefore, cannot guarantee or imply reliability, serviceability,
//  or function of these programs.
//  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
//  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
//  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
//  FOR A PARTICULAR PURPOSE.
//************************************************************************/
/*END DISCLAIMER */

package com.ibm.workflow.servlet.sample;

import java.io.*;
import java.security.Security;
import java.security.PublicKey;
import java.security.Provider;
import java.security.NoSuchAlgorithmException;
import java.security.cert.*;
import javax.servlet.http.*;

import com.ibm.workflow.servlet.client.*;
import com.ibm.workflow.api.ExecutionService;
import com.ibm.workflow.api.FmcException;
import com.ibm.workflow.api.ServicePackage.*;

/**
* This class demonstrates third party authentication.
*/
public class AuthenticationHandler extends GenericCommandHandler
{
  public static class FmcProvider extends Provider
  {
    public FmcProvider()
    {
      super("MQWF", 0.9, "MQWF Security Provider V0.9");
      put("CertificateFactory.MQWF",
          "com.ibm.workflow.servlet.sample.AuthenticationHandler$FmcCertificateFactory");
    }
  }

  public static class FmcCertificateFactory extends CertificateFactorySpi
  {
    public Certificate engineGenerateCertificate(InputStream is)
                       throws CertificateException
    {
      try
      {
        DataInputStream dis = new DataInputStream(is);
        byte[] encoded = new byte[dis.available()];
        dis.readFully(encoded);
        return FmcCertificate.decode(encoded);
      }
      catch (IOException xcpt)
      {
        throw new CertificateParsingException(xcpt.getMessage());
      }
    }

    public java.util.Collection engineGenerateCertificates(InputStream is)
                                throws CertificateException
    {
      throw new CertificateException("generateCertificates not supported");
    }

    public CRL engineGenerateCRL(InputStream is)
               throws CRLException
    {
      throw new CRLException("generateCRL not supported");
    }

    public java.util.Collection engineGenerateCRLs(InputStream is)
                                throws CRLException
    {
      throw new CRLException("generateCRLs not supported");
    }
  }

  public static class FmcCertificate extends Certificate
  {
    public FmcCertificate()
    {
      super("MQWF");
    }

    public FmcCertificate(String userID, String password)
    {
      super("MQWF");
      this.userID = userID;
      this.password = password;
    }

    public String getUserID()
    {
      return userID;
    }

    public String getPassword()
    {
      return password;
    }

    public byte[] getEncoded() throws CertificateEncodingException
    {
      if (userID == null || password == null)
        throw new CertificateEncodingException("Certificate is null");

      try
      {
        return new String(userID + "\n" + password).getBytes("UTF8");
      }
      catch (UnsupportedEncodingException xcpt)
      {
        throw new CertificateEncodingException(xcpt.getMessage());
      }
    }

    static FmcCertificate decode(byte[] encoded)
                          throws CertificateException
    {
      try
      {
        String s = new String(encoded, "UTF8");
        int pos = s.indexOf('\n');

        return new FmcCertificate(s.substring(0, pos),
                                  s.substring(pos + 1));
      }
      catch (UnsupportedEncodingException xcpt)
      {
        throw new CertificateEncodingException(xcpt.getMessage());
      }
      catch (ArrayIndexOutOfBoundsException xcpt)
      {
        throw new CertificateParsingException(xcpt.getMessage());
      }
    }

    public PublicKey getPublicKey()
    {
      return null;
    }

    public String toString()
    {
      return "FmcCertificate for " + userID;
    }

    public void verify(PublicKey key) throws NoSuchAlgorithmException
    {
      throw new NoSuchAlgorithmException("verify not supported");
    }

    public void verify(PublicKey key, String sigProvider)
                throws NoSuchAlgorithmException
    {
      throw new NoSuchAlgorithmException("verify not supported");
    }

    private String userID;
    private String password;
  }

  public void init(Config config)
  {
    super.init(config);

    try
    {
      new FmcCertificate(); // getMethod does not work since it's protected
      hasCertificate = true;
      Security.addProvider(new AuthenticationHandler.FmcProvider());
      getContext().log("      AuthenticationHandler: Added security provider FmcProvider", false);
    }
    catch (NoClassDefFoundError xcpt)
    {
      getContext().log("      AuthenticationHandler: Java2 required - " + xcpt.toString(), true);
    }
    catch (NoSuchMethodError xcpt)
    {
      getContext().log("      AuthenticationHandler: wrong Certificate - " + xcpt.toString(), true);
    }
  }

  /**
  * Returns information about the handler, such as author, version,
  * and copyright.
  * @return a String containing handler information
  */
  public String getHandlerInfo()
  {
    return "MQWF Sample - Authentication exit V0.9";
  }

  public byte[] getCredentials(HttpServletRequest request)
                throws ClientException
  {
    Certificate cert = null;

    /* Servlet 2.2 specs - 5.7 SSL Attributes                              */
    /* --------------------------------------                              */
    /* If a request has been transmitted over a secure protocol, such as   */
    /* HTTPS, this information must be exposed via the isSecure method of  */
    /* the ServletRequest interface.                                       */
    /*                                                                     */
    /* In servlet containers that are running in a Java 2 Standard         */
    /* Edition, v 1.2 or Java 2 Enterprise Edition, v 1.2 environment, if  */
    /* there is an SSL certificate associated with the request, it must be */
    /* exposed to the servlet programmer as an array of objects of type    */
    /* java.security.cert.X509Certificate and accessible via a             */
    /* ServletRequest attribute of javax.servlet.request.X509Certificate.  */
    /*                                                                     */
    /* For a servlet container that is not running in a Java2 Standard     */
    /* Edition 1.2 environment, vendors may provide vendor specific        */
    /* request attributes to access SSL certificate information.           */

    if (cert == null) // try servlet 2.2 standard
    {
      X509Certificate certChain[] =
         (X509Certificate[]) request.getAttribute("javax.servlet.request.X509Certificate");

      if (certChain != null)
        cert = certChain[0];
    }

    if (cert == null) // try IBM WebSphere
    {
      X509Certificate certChain[] =
         (X509Certificate[]) request.getAttribute("javax.net.ssl.peer_certificates");

      if (certChain != null)
        cert = certChain[0];
    }

    String authcertWithNL = request.getHeader("auth-cert");

    if (authcertWithNL != null) // try Allaire JRun (?)
    {
      String authcert = "";

      try
      {
        BufferedReader buffer =
           new BufferedReader(new StringReader(authcertWithNL));
        for (String line; (line = buffer.readLine()) != null; )
          authcert = authcert.concat(line);
      }
      catch (IOException xcpt)
      {
        throw new ClientException(request, xcpt);
      }

      try
      {
        InputStream is = new ByteArrayInputStream(authcert.getBytes());
        is = new DataInputStream(is);
        CertificateFactory cf = CertificateFactory.getInstance("X.509");
        cert = cf.generateCertificate(is);
      }
      catch (CertificateException xcpt)
      {
        throw new ClientException(request, xcpt);
      }
    }

    // fallback - this dummy certificate is the only one supported by
    // the current implementation of logon3()
    if (cert == null && hasCertificate)
    {
      String userID = request.getParameter("userID");
      String password = request.getParameter("password");

      cert = new FmcCertificate(userID, password);
    }

    try
    {
      if (cert != null)
        return cert.getEncoded();
    }
    catch (CertificateException xcpt)
    {
      throw new ClientException(request, xcpt);
    }

    return null;
  }

  public void logon3(ExecutionService service, byte[] credentials,
                     SessionMode mode, AbsenceIndicator absenceIndicator)
              throws FmcException
  {
    String userID = null;
    String password = null;

    try
    {
      InputStream is = new ByteArrayInputStream(credentials);
      CertificateFactory cf = CertificateFactory.getInstance("MQWF");
      FmcCertificate cert = (FmcCertificate) cf.generateCertificate(is);
      userID = cert.getUserID();
      password = cert.getPassword();
    }
    catch (CertificateException xcpt)
    {
      getContext().log("Error creating certificate", xcpt);
      // let logon2 handle missing parameters...
    }

    service.logon2(userID, password, mode, absenceIndicator);
  }

  private boolean hasCertificate = false;
}
