/*BEGIN CMVC */
//*************************************************************************
//
//  Workfile: servlet/client/EnhancedJSPViewer.java, v3-java, v360
//  Last update: 07/05/21 13:25:40
//  SCCS path, id: /home/flowmark/vc/0/6/5/2/s.64 1.12
//
//************************************************************************/
/*END CMVC */
/*BEGIN COPYRIGHT */
//*****************************************************************************
//
//  Licensed Materials - Property of IBM
//  5697-FM3 5655-A96
//  (C) Copyright IBM Corp. 1999, 2008
//  US Government Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract
//  with IBM Corp.
//
//****************************************************************************/
//END COPYRIGHT */
/*BEGIN DISCLAIMER */
//*************************************************************************
//  DISCLAIMER
//  ----------
//  This material contains programming source code for your consideration.
//  These examples have not been thoroughly tested under all conditions.
//  IBM, therefore, cannot guarantee or imply reliability, serviceability,
//  or function of these programs.
//  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
//  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
//  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
//  FOR A PARTICULAR PURPOSE.
//************************************************************************/
/*END DISCLAIMER */

package com.ibm.workflow.servlet.client;

import java.util.*;
import java.io.File;
import java.io.IOException;

import javax.servlet.http.HttpServletRequest;

import com.ibm.workflow.api.*;

/**
* <p>This Viewer should be used when your Application server supports the
* JSP V1.0 standard. It uses JSPs for custom process and activity forms as
* well as for some other commands. This provides much more flexibility than
* the propriatary HTML template files used by <code>DefaultViewer</code> and
* it also supports internationalization.</p>
* <p>You must set
* <code>DefaultViewer=com.ibm.worklflow.servlet.client.JSPViewer</code> in the
* <a href="../../../../../../properties.html#Viewer">WebClient.properties</a>
* file to in order to enable this <code>Viewer</code>.</p>
*/
public class EnhancedJSPViewer implements Viewer {

    public void init(Config config)
    {
        //DV removal super.init(config);
        enableTemplatelists =
        config.getParameter("DefaultViewer", "EnableTemplatelists", true);
        enableInstancelists =
        config.getParameter("DefaultViewer", "EnableInstancelists", true);
        enableWorklists =
        config.getParameter("DefaultViewer", "EnableWorklists", true);
        enableLists =
        config.getParameter("DefaultViewer", "EnableLists", true);
        
        enableActivityInstanceLists = config.getParameter("DefaultViewer", "EnableActivityInstancelists", true);

        autoRefresh = config.getParameter("DefaultViewer", "AutoRefresh", false);

        pageAfterLogoff = config.getParameter("DefaultViewer", "PageAfterLogoff",
                                              "{_HTMLDir_}/Logon.html");
        pageAfterLogoff = config.getContext().getURL(pageAfterLogoff);

        generateForms = config.getParameter("DefaultViewer", "GenerateForms", true);
    }



    /**
    * Creates the page that is shown in case of an error. This method implements
    * 'logon splicing'. In the case of an expired session, the
    * original command will be remembered and executed after a successful
    * re-logon.
    * @param context The request context.
    * @return The response page.
    */
    public ResponsePage errorResponse(RequestContext context)
    {
        ClientException error = context.getException();
        HttpServletRequest request = error.getRequest();
        String command = request == null ? null : request.getParameter("command");

        if (command == null || !(error instanceof NoSessionException)) {
            Throwable nested = error.getNestedException();

            if (nested instanceof FmcException) {
                FmcException xcpt = (FmcException) nested;

                if (xcpt.rc == FmcException.FMC_ERROR_COMMUNICATION && "logon".equals(command)) {
                    // communication error during logon -  MQWF server not started?
                    error = new ClientException(request,xcpt, context.get("Error.NoConnection"));
                    context.setException(error);
                } else
                    if (xcpt.rc == FmcException.FMC_ERROR_QRY_RESULT_TOO_LARGE) {
                    // show list properties to allow changing the threshold
                    try {
                        context.setKindOfList(SessionContext.LISTS);
                        context.setMessage(context.get("Message.TooManyItems"));
                        ResponsePage page = showListPropertiesResponse(context);

                        if (page != null)
                            return page;
                    } catch (ClientException ignored) {
                    }

                }
            }


            String message = Util.getMessage(error);
            String query = ""; // for E-Help feature
            String errorNo = "";

            if (error.getNestedException() != null) {
                if (error.getNestedException() instanceof com.ibm.workflow.api.FmcException) {
                    FmcException ex = (FmcException) error.getNestedException();

                    int start=ex.messageText.indexOf("FMC");
                    int stop =ex.messageText.indexOf(" ", start);
                    errorNo = ex.messageText.substring(start, stop);
                    query ="IBM WebSphere MQ Workflow " +" "+ ex.messageText.substring(start, stop);
                    int startmq = ex.messageText.indexOf("Reason Code: "); // check if there is some underlying
                                                                           // information on MQ errors
                    if (startmq > 0) {
                        //Get MQ error code in case of 'communication errors'
                        //Error message: Communication error - Failing Action: %1$s, Reason Code: %2$s, Failing Object: %3$s.
                        int stopmq = ex.messageText.indexOf(",",startmq);
                        String mqReason = ex.messageText.substring(startmq+13, stopmq); //remove 'Reason Code: '
                        query = query + " " + mqReason;
                        errorNo = errorNo + "+" + mqReason;
                    }

                    int startdb = ex.messageText.indexOf("SQL code: "); // check for underlying DB errors

                    if (startdb >0) {
                        int stopdb = ex.messageText.indexOf(".",startdb);
                        String dbReason = ex.messageText.substring(startdb+10, stopdb); //remove 'SQL code: '
                        query = query + " " + dbReason;
                        errorNo = errorNo + "+" + dbReason;
                    }

                } else
                    query = error.getNestedException().getMessage();
            } else {
                query = error.getMessage();
            }
            
            message = context.toHTMLText(message,"");
            message = Util.replaceInString(message, "\n", "<br>\r\n");
            String stack = ClientException.getStack(error);
            stack =context.toHTMLText(stack,"");
            stack = Util.replaceInString(stack, "\n", "<br>\r\n");
            String errorTitle = RequestContext.toHTMLText(error.getTitle(),error.getTitle());
            if (errorTitle != null) context.setAttribute("_Title_", errorTitle);
            if (message != null)context.setAttribute("_Message_", message);
            if (stack != null) context.setAttribute("_StackTrace_", stack);

            if (query != null)context.setAttribute("SearchString", RequestContext.toHTMLText(query, query));
            if (errorNo !=null)context.setAttribute("ErrorNo", RequestContext.toHTMLText(errorNo, errorNo));

            return new ResponsePage(context, "/forms/Error.jsp", -1);

        }

        // doing a re-logon just for logging of? Dunno...
        if ("logoff".equals(command))
            return logoffResponse(context);

        LogonContext ctx = new LogonContext(context);
        ctx.setMessage(context.get("Message.ReLogon"));
        ctx.addParameter("command2", command);

        Enumeration e = request.getParameterNames();

        while (e.hasMoreElements()) {
            String parm = (String) e.nextElement();

            if (!"command".equals(parm))
                ctx.addParameter(parm, request.getParameter(parm));
        }

        return new ResponsePage(ctx, "/Logon.jsp", -1);
    }

    /**
    * Creates the page that is displayed after a successful logon.
    * If this was a spliced logon page (see {@link #errorResponse}), the
    * remembered command is extracted and executed. Otherwise, the first 
    * list is displayed. If no list is found, the
    * page to create a new list will be returned.
    * @param context The request context.
    * @return The response page.
    * @exception ClientException if an error occurred
    */
    public ResponsePage logonResponse(RequestContext context)
    throws ClientException
    {
        BuiltinHandler builtin = context.getBuiltinHandler();

        HttpServletRequest request = context.getRequest();
        String command = request == null ? null : request.getParameter("command2");

        if (command != null)
            return(ResponsePage) builtin.execute(command, request, null, true);

        return mostRecentList(context, false);
    }

    /**
    * Creates the page that is shown after logoff.
    * This request will be forwarded to <code>Logon.jsp</code>.
    * @param context The request context.
    * @return The response page.
    */
    public ResponsePage logoffResponse(RequestContext context)
    {
        return new ResponsePage(new LogonContext(context), "/Logon.jsp", -1);
    }

    /**
    * Creates the page that displays a list of activity instances.
    * This request will be forwarded to <code>forms/ActivityList.jsp</code>
    * which dynamically creates a page that contains a table with a row
    * for each activity on the list. This table also contains buttons that
    * allow to invoke commands on the list entries
    * @param context The request context.
    * @return The response page or <code>null</code> if activity lists are
    *         disabled and the Command sent is not 'createPersistentList'. If
    *         process instance lists are disabled and the Command is 'createPersistentList'
    *         the list of lists will be displayed.
    * @exception ClientException if an error occurred
    */
    public ResponsePage queryActivityInstancesResponse(RequestContext context)
    throws ClientException
    {
        if (!enableActivityInstanceLists){
        	if ("createPersistentList".equals(context.getRequestParameter("command",null)))
            	return redirectResponse(context,"forms/ListofList.jsp",Command.LIST_QUERY_LISTS,null,null);
            else
            	return null;
        }
        context.setAttribute("navigatorEntries", createNavigatorEntries(context, false));
        try {
            return redirectResponse(context, "/forms/ActivityList.jsp",
                                    Command.AIL_QUERY_INSTANCES, context.getActivityInstanceList().persistentOid(), null);
        } catch (FmcException ex) {
            throw new ClientException(context, ex);
        }
    }


    /**
    * Creates the page that displays the list of lists.
    * This request will be forwarded to <code>forms/ListofLists.jsp</code>
    * which dynamically creates a page that contains a table with a row
    * for each list on the list. This table also contains buttons that
    * allow to invoke other commands, such as deleting a list.
    * @param context The request context.
    * @return The response page or <code>null</code> if lists of lists are
    *         disabled
    * @exception ClientException if an error occurred
    */
    public ResponsePage queryListsResponse(RequestContext context)
    throws ClientException
    {
        if (!enableLists)
            return null;

        context.setAttribute("navigatorEntries", createNavigatorEntries(context, false));

        return redirectResponse(context, "/forms/ListofLists.jsp",
                                Command.LIST_QUERY_LISTS, null, null);
    }

    /**
    * Creates the page that displays list settings.
    * This request will be forwarded to <code>forms/ListPropertiesViewer.jsp</code>.
    * @param context The request context.
    * @return The response page.
    * @exception ClientException if an error occurred
    */
    public ResponsePage showListPropertiesResponse(RequestContext context)
    throws ClientException
    {
        return new ResponsePage(context, "/forms/ListPropertiesViewer.jsp");
    }

    /**
    * Creates the page that displays the list of lists.
    * This request will be forwarded to <code>forms/ListofLists.jsp</code>
    * which dynamically creates a page that contains a table with a row
    * for each list.
    * @param context The request context.
    * @return The response page.
    * @exception ClientException if an error occurred
    */
    public ResponsePage setListPropertiesResponse(RequestContext context)
    throws ClientException
    {
        context.setAttribute("navigatorEntries", createNavigatorEntries(context, false));

        return redirectResponse(context, "/forms/ListofLists.jsp",
                                Command.LIST_QUERY_LISTS, null, null);
    }

    /**
    * @doc
    */
    public ResponsePage createListResponse(RequestContext context)
    throws ClientException
    {
        return new ResponsePage(context, "/forms/ListPropertiesViewer.jsp");
    }

    /**
    * Creates the page that displays a process template list.
    * This request will be forwarded to <code>forms/ProcessTemplateList.jsp</code>
    * which dynamically creates a page that contains a table with a row
    * for each template on the list. This table also contains buttons that
    * allow to invoke other commands, such as displaying a template's properties.
    * @param context The request context.
    * @return The response page or <code>null</code> if template lists are
    *         disabled and the Command sent is not 'createPersistentList'. If
    *         process instance lists are disabled and the Command is 'createPersistentList'
    *         the list of lists will be displayed.
    * @exception ClientException if an error occurred
    */
    public ResponsePage queryProcessTemplatesResponse(RequestContext context)
    throws ClientException
    {
        if (!enableTemplatelists){
        	if ("createPersistentList".equals(context.getRequestParameter("command",null)))
            	return redirectResponse(context,"forms/ListofList.jsp",Command.LIST_QUERY_LISTS,null,null);
            else
            	return null;
        }
        try {
            context.setAttribute("navigatorEntries", createNavigatorEntries(context, false));

            return redirectResponse(context, "/forms/ProcessTemplateList.jsp", Command.PTL_QUERY_TEMPLATES,
                                    context.getTemplateList().persistentOid(), null);
        } catch (FmcException xcpt) {
            throw new ClientException(context, xcpt);
        }
    }

    /**
    * Creates the page that displays process template properties.
    * This request will be forwarded to <code>forms/PropertiesViewer.jsp</code>.
    * @param context The request context.
    * @return The response page.
    * @exception ClientException if an error occurred
    */
    public ResponsePage showTemplatePropertiesResponse(RequestContext context)
    throws ClientException
    {
        return new ResponsePage(context, "/forms/PropertiesViewer.jsp");
    }

    /**
    * Creates the page displayed after a new process instance has been created
    * and started.
    * <p>If the process instance was not yet started because it
    * <a href="BuiltinHandler.html#data_reqd">required input data</a>, the
    * request will be forwarded to <code>processes/<i>TemplateName</i>.jsp</code>.
    * This JSP should contain a form to fill in the required input container
    * data and then again call {@link BuiltinHandler#createAndStartInstance}.
    * For a sample, see the <a href="../../../../../../samples.html#credit">
    * credit request process</a>. If the JSP file could not be found, the
    * client tries to load file <code>processes/<i>TemplateName</i>.html</code>
  * which may contain the following substitution variables:
  * <table border="1" cellspacing="0" cellpadding="4">
  *   <tr>
  *     <th>Name</th>
  *     <th>Replacement</th>
  *   </tr>
  *   <tr>
  *     <td>{_HTMLDir_}</td>
  *     <td>The root URI of the servlet's web application, for example,
  *         <code>/MQWFClient</code>.</td>
  *   </tr>
  *   <tr>
  *     <td>{_MQWFServlet_}</td>
  *     <td>The servlet's name, for example,
  *         <code>/MQWFClient/servlet/Main</code>.</td>
  *   </tr>
  *   <tr>
  *     <td>{_ActionArea_}</td>
  *     <td>A form containing a <b>Submit</b> button which calls
  *         <code>createAndStartInstance</code> again with the
  *         <a href="BuiltinHandler.html#data_avail">submitData</a>
  *         request parameter set and with the input data entered, and
  *         a <b>Cancel</b> button which calls <code>queryProcessTemplates</code>
  *         to go back to the template list.</td>
  *   </tr>
  * </table>
  * If the HTML template file cannot be found, and
  * <a href="../../../../../../properties.html#generate">GenerateForms</a>
  * is set to true, a form is created from the process' input container.
  * Submitting this form will start the instance with the data entered in
  * the generated form.</p>
  * <p>If the process instance was started from a template list, this
  * template list will be displayed next.</p>
  * <p>If the process instance was started by the web user, the page denoted
  * by the <code>SuccessLocation</code> request parameter will be sent as
  * response. If <code>SuccessLocation</code> is not set, the default form
  * <code>forms/ProcessStarted.html</code> will be used. If additionally the
  * <code>UseRedirect</code> request parameter is set to <code>true</code>, the
  * request will be redirected to that page. If <code>UseRedirect</code> is
  * <code>false</code> or not set, the page
  * is loaded as an HTML template which may contain the following substitution
  * variables:
  * <table border="1" cellspacing="0" cellpadding="4">
  *   <tr>
  *     <th>Name</th>
  *     <th>Replacement</th>
  *   </tr>
  *   <tr>
  *     <td>{_HTMLDir_}</td>
  *     <td>The root URI of the servlet's web application, for example,
  *         <code>/MQWFClient</code>.</td>
  *   </tr>
  *   <tr>
  *     <td>{_MQWFServlet_}</td>
  *     <td>The servlet's name, for example,
  *         <code>/MQWFClient/servlet/Main</code>.</td>
  *   </tr>
  *   <tr>
  *     <td>{_TemplateName_}</td>
  *     <td>The name of the process template.</td>
  *   </tr>
  *   <tr>
  *     <td>{_InstanceName_}</td>
  *     <td>The name of the process instance that has been started.</td>
  *   </tr>
  * </table>
  * </p>
    * <p>If the process instance was started by the web user, the request will be
    * forwarded to the page denoted by the <code>SuccessLocation</code> request
    * parameter if it is a JSP. Note that this JSP must be in the same Web Application
    * as the page containing the <code>createAndStartInstance</code> command. If it
    * is not, you must additionally set the <code>UseRedirect</code> request parameter
    * to true. In this case, the JSP will not have access to the {@link RequestContext}
    * bean.</p>
    *
    * @param context The request context.
    * @return The response page.
    * @exception ClientException if an error occurred
    */
    public ResponsePage createAndStartInstanceResponse(RequestContext context)
    throws ClientException
    {
        try {
            if (context.getInstance() == null) {
                // not yet started because input data required
                String templateName = context.getTemplate().name();
                String urlName = "/processes/" + templateName + ".jsp";

                File jspFile = new File(context.getDocumentRoot(), urlName);

                if (jspFile.isFile())
                    return new ResponsePage(context, urlName);

                context.log(jspFile + " not found, trying DefaultViewer", false);
            } else if (context.getRequestParameter("id", null) == null) {
                // process was started by the webuser
                String urlname = context.getRequestParameter("SuccessLocation", null);
                String useRedirect = context.getRequestParameter("UseRedirect", Boolean.FALSE.toString());

                if (!Util.booleanValue(useRedirect, false) && urlname != null && urlname.endsWith(".jsp")) {
                    // JSPs should not be treated as HTML templates like
                    // DefaultViewer does
                    return new ResponsePage(context, context.getURL(urlname));
                }
            }

            return getHTMLTemplateForCreateAndStart(context);
        } catch (FmcException xcpt) {
            throw new ClientException(context, xcpt);
        }
    }

    /**
    * Creates the page that displays a process instance list.
    * This request will be forwarded to <code>forms/ProcessInstanceList.jsp</code>
    * which dynamically creates a page that contains a table with a row
    * for each instance on the list. This table also contains buttons that
    * allow to invoke other commands, such as displaying an instance's properties.
    * @param context The request context.
    * @return The response page or <code>null</code> if instance lists are
    *         disabled and the Command sent is not 'createPersistentList'. If
    *         process instance lists are disabled and the Command is 'createPersistentList'
    *         the list of lists will be displayed.
    * @exception ClientException if an error occurred
    */
    public ResponsePage queryProcessInstancesResponse(RequestContext context)
    throws ClientException
    {
        if (!enableInstancelists){
            if ("createPersistentList".equals(context.getRequestParameter("command",null)))
            	return redirectResponse(context,"forms/ListofList.jsp",Command.LIST_QUERY_LISTS,null,null);
            else
            	return null;
        }
        try {
            context.setAttribute("navigatorEntries", createNavigatorEntries(context, false));

            return redirectResponse(context, "/forms/ProcessInstanceList.jsp", Command.PIL_QUERY_INSTANCES,
                                    context.getInstanceList().persistentOid(), null);
        } catch (FmcException xcpt) {
            throw new ClientException(context, xcpt);
        }
    }

    /**
    * Creates the page that displays process instance properties.
    * This request will be forwarded to <code>forms/PropertiesViewer.jsp</code>.
    * @param context The request context.
    * @return The response page.
    * @exception ClientException if an error occurred
    */
    public ResponsePage showInstancePropertiesResponse(RequestContext context)
    throws ClientException
    {
        return new ResponsePage(context, "/forms/PropertiesViewer.jsp");
    }

    /**
    * Creates the page that will be displayed after a process instance has
    * been started.
    * If the process instance was actually started, base class'
    * {@link DefaultViewer#startInstanceResponse startInstanceResponse} method
    * will be called. If the process instance was not yet started because it
    * required input data, the request will be forwarded to
    * <code>processes/<i>TemplateName</i>.jsp</code>, if this file exists.
    * @param context The request context.
    * @return The response page.
    * @exception ClientException if an error occurred
    */
    public ResponsePage startInstanceResponse(RequestContext context)
    throws ClientException
    {
        try {
            ProcessInstance instance = context.getInstance();

            if (instance.state().value() ==
                com.ibm.workflow.api.ProcessInstancePackage.ExecutionState._READY) {
                // not yet started because input data required
                String templateName = instance.processTemplateName();
                String urlName = "/processes/" + templateName + ".jsp";

                File jspFile = new File(context.getDocumentRoot(), urlName);

                if (jspFile.isFile())
                    return new ResponsePage(context, urlName);

                context.log(jspFile + " not found, trying DefaultViewer", false);
            }

            return getStartInstanceResponseHTMLfile(context);
        } catch (FmcException xcpt) {
            throw new ClientException(context, xcpt);
        }
    }

    /**
    * Creates the page that is displayed after a process instance has been
    * suspended. If the process instance was actually suspended, the most recent
    * list will be displayed. If it has not yet been suspended because the
    * necessary data has not been specified, the request will be forwarded
    * to <code>forms/SuspendInstance.jsp</code> so the parameters for the
    * {@link BuiltinHandler#suspendInstance suspendInstance} command can be
    * specified.
    * @param context The request context.
    * @return The response page.
    * @exception ClientException if an error occurred
    */
    public ResponsePage suspendInstanceResponse(RequestContext context)
    throws ClientException
    {
        if (context.getRequestParameter("submitData", null) != null) {
            return mostRecentList(context, !autoRefresh);
        }

        return new ResponsePage(context, "/forms/SuspendInstance.jsp");
    }

    /**
    * Creates the page that displays activity instance properties.
    * This request will be forwarded to <code>forms/PropertiesViewer.jsp</code>.
    * @param context The request context.
    * @return The response page.
    * @exception ClientException if an error occurred
    */
    public ResponsePage showActivityPropertiesResponse(RequestContext context)
    throws ClientException
    {
        return new ResponsePage(context, "/forms/PropertiesViewer.jsp");
    }

    /**
    * Creates the page that is displayed after an activity instance has been
    * force restarted. If the activity instance was actually force restarted, the
    * request will be forwarded to <code>forms/PropertiesViewer.jsp</code> (this
    * is the only page from where activities can be force restarted). If it has
    * not yet been force restarted because the necessary data has not been
    * specified, the request will be forwarded to
    * <code>forms/ForceRestart.jsp</code> so the parameters for the
    * {@link BuiltinHandler#forceRestartActivity forceRestartActivity} command
    * can be specified.
    * @param context The request context.
    * @return The response page.
    * @exception ClientException if an error occurred
    */
    public ResponsePage forceRestartActivityResponse(RequestContext context)
    throws ClientException
    {
        if (context.getRequestParameter("submitData", null) == null)
            return new ResponsePage(context, "/forms/ForceRestart.jsp");
        if (context.getRequestParameter("list",null) == null)
            return new ResponsePage(context, "/forms/PropertiesViewer.jsp");

        return mostRecentList(context, !autoRefresh);
    }

    /**
    * Creates the page that is displayed after an activity instance has been
    * force finished. If the activity instance was actually force finished, the
    * request will be forwarded to <code>forms/PropertiesViewer.jsp</code> if
    * the action was called from the properties page. Otherwise it was called from
    * an activity instance page. In this case the folowing page is the list itself.
    * If it has not yet been force finished because the necessary data has not been
    * specified, the request will be forwarded to
    * <code>forms/ForceFinish.jsp</code> so the parameters for the
    * {@link BuiltinHandler#forceFinishActivity forceFinishActivity} command
    * can be specified.
    * @param context The request context.
    * @return The response page.
    * @exception ClientException if an error occurred
    */
    public ResponsePage forceFinishActivityResponse(RequestContext context)
    throws ClientException
    {
        if (context.getRequestParameter("submitData", null) == null)
            return new ResponsePage(context, "/forms/ForceFinish.jsp");
        if (context.getRequestParameter("list",null) == null){
            
        	/*
        	 * The following check is required to avoid problems in cases
        	 * where the activity instance that was force finished does no longer
        	 * exist, e.g. if the last activity of a process instance that is
        	 * automatically deleted on completion is 'force-finished'.        	  
        	 */
        	
        	try {
        		ActivityInstance ai = context.getActivityInstance();
        		if (ai != null){
        		  ai.refresh();
        		  return new ResponsePage(context, "/forms/PropertiesViewer.jsp");
        		}
        		else return mostRecentList(context, false); //refresh in any case
        	}
        	catch (FmcException ex){
        		if (ex.rc == FmcException.FMC_ERROR_DOES_NOT_EXIST){
        		  context.log("EnhancedJSPViewer.forceFinishActivityResponse(): Activity is no longer valid. Continuing with most recent list", false);
        		}
        		else {
        			context.log("EnhancedJSPViewer.forceFinishActivityResponse(): Error occured. going to error page", false);
        			throw new ClientException(context,ex,"Error refreshing activity instance");
        		}
        		return mostRecentList(context, false); //refresh in any case to get updated list
        	}
        	
        }

        return mostRecentList(context, !autoRefresh);
    }

    /**
    * Creates the page that is displayed after an activity instance has been
    * terminated. This request will be forwarded to
    * <code>forms/PropertiesViewer.jsp</code> (this is the only page from where
    * activities can be terminated).
    * @param context The request context.
    * @return The response page.
    * @exception ClientException if an error occurred
    */
    public ResponsePage terminateActivityResponse(RequestContext context)
    throws ClientException
    {
        if ("MQWFActivityList".equals(context.getRequest().getParameter("list")))
            return mostRecentList(context, autoRefresh);
        else
            return new ResponsePage(context, "/forms/PropertiesViewer.jsp");
    }

    /**
    * Creates the page that displays control connector properties.
    * This request will be forwarded to <code>forms/PropertiesViewer.jsp</code>.
    * @param context The request context.
    * @return The response page.
    * @exception ClientException if an error occurred
    */
    public ResponsePage showControlConnectorPropertiesResponse(RequestContext context)
    throws ClientException
    {
        return new ResponsePage(context, "/forms/PropertiesViewer.jsp");
    }

    /**
    * Creates the page that displays a worklist.
    * This request will be forwarded to <code>forms/WorkitemList.jsp</code>
    * which dynamically creates a page that contains a table with a row
    * for each work item on the list. This table also contains buttons that
    * allow to invoke other commands, such as displaying an item's properties.
    * @param context The request context.
    * @return The response page or <code>null</code> if workitem lists are
    *         disabled and the Command sent is not 'createPersistentList'. If
    *         process instance lists are disabled and the Command is 'createPersistentList'
    *         the list of lists will be displayed.
    * @exception ClientException if an error occurred
    */
    public ResponsePage queryWorkItemsResponse(RequestContext context)
    throws ClientException
    {
        if (!enableWorklists){
        	if ("createPersistentList".equals(context.getRequestParameter("command",null)))
            	return redirectResponse(context,"forms/ListofList.jsp",Command.LIST_QUERY_LISTS,null,null);
            else
            	return null;
        }

        try {
            context.setAttribute("navigatorEntries", createNavigatorEntries(context, false));

            return redirectResponse(context, "/forms/WorkitemList.jsp", Command.WL_QUERY_WORK_ITEMS,
                                    context.getWorkList().persistentOid(), null);
        } catch (FmcException xcpt) {
            throw new ClientException(context, xcpt);
        }
    }

    /**
    * Creates the page that displays work item properties.
    * This request will be forwarded to <code>forms/PropertiesViewer.jsp</code>.
    * @param context The request context.
    * @return The response page.
    * @exception ClientException if an error occurred
    */
    public ResponsePage showWorkItemPropertiesResponse(RequestContext context)
    throws ClientException
    {
        return new ResponsePage(context, "/forms/PropertiesViewer.jsp");
    }

    /**
    * Creates the page that will be displayed when a work item has been
    * checked out.
    * <p>This request will be forwarded to
    * <code>programs/<i>ProgramName</i>.jsp</code> (where <i>ProgramName</i>
    * is the name of the program assigned to the activity). This JSP should
    * contain a form to fill in the required input container
    * data and then call {@link BuiltinHandler#checkInWorkItem}.
    * For a sample, see the <a href="../../../../../../samples.html#credit">
    * credit request process</a>. If the JSP file could not be found, the
    * request will try to load <code>programs/<i>ProgramName</i>.html</code> (where <i>ProgramName</i>
  * is the name of the program assigned to the activity). This HTML template
  * file may contain the following substitution variables:
  * <table border="1" cellspacing="0" cellpadding="4">
  *   <tr>
  *     <th>Name</th>
  *     <th>Replacement</th>
  *   </tr>
  *   <tr>
  *     <td>{_HTMLDir_}</td>
  *     <td>The root URI of the servlet's web application, for example,
  *         <code>/MQWFClient</code>.</td>
  *   </tr>
  *   <tr>
  *     <td>{_MQWFServlet_}</td>
  *     <td>The servlet's name, for example,
  *         <code>/MQWFClient/servlet/Main</code>.</td>
  *   </tr>
  *   <tr>
  *     <td>{_Date_}</td>
  *     <td>The current date.</td>
  *   </tr>
  *   <tr>
  *     <td>{_Time_}</td>
  *     <td>The current time.</td>
  *   </tr>
  *   <tr>
  *     <td>{_Name_}</td>
  *     <td>The name of the work item.</td>
  *   </tr>
  *   <tr>
  *     <td>{_Description_}</td>
  *     <td>The description of the work item.</td>
  *   </tr>
  *   <tr>
  *     <td>{_UserID_}</td>
  *     <td>The owner of the work item.</td>
  *   </tr>
  *   <tr>
  *     <td>{_ActionArea_}</td>
  *     <td>A form containing a <b>Submit</b> button which calls
  *         <code>checkInWorkItem</code> with the input data entered and
  *         a <b>Cancel</b> button which calls <code>cancelWorkItem</code>.</td>
  *   </tr>
  * </table>
  * If the HTML template file cannot be found, and
  * <a href="../../../../../../properties.html#generate">GenerateForms</a>
  * is set to true, a form is created from the work item's input container.
  * Submitting this form will check in the work item with the data entered in
  * the generated form.
    * @param context The request context.
    * @return The response page.
    * @exception ClientException if an error occurred
    */
    public ResponsePage checkOutWorkItemResponse(RequestContext context)
    throws ClientException
    {
        try {
            WorkItem workItem = context.getWorkItem();
            Container container = context.getContainer();
            String urlName;
            File jspFile;
            if (container != null){
            	
              urlName = "/programs/"+container.getString("_PROCESS_MODEL")+"/"+workItem.implementation()+".jsp";
              jspFile = new File(context.getDocumentRoot(), urlName);
              if (jspFile.isFile()) return new ResponsePage(context, urlName);
            }
            else {
            	if (context.isDebugEnabled()){
            		context.log("Container is null in EnhancedJSPViewer.checkOutWorkItemResponse -> WI has been started not checked out",false);
            	}
            	return mostRecentList(context,!autoRefresh);
            }
            // could not find JSP in the template folder -> Trying programs directly
            urlName = "/programs/" + workItem.implementation() + ".jsp";

            jspFile = new File(context.getDocumentRoot(), urlName);

            if (jspFile.isFile()) {
                // EF: Netscape reloads pages with immediate expiry even when only
                // the browser window is resized or printed. The idea was to set
                // the page expiry to 5 minutes for work items to avoid those
                // annoying 'already checked out' exceptions (note that this browser
                // behavior is not a problem for most other requests like showing the
                // worklist, ...). But this workaround is no good when the checkout
                // is cancelled. Because then the next 'checkout' request is satisfied
                // from the browser cache without actually checking out the work item -
                // resulting in a 'state does not allow this action' exception when
                // trying to check in the work item...

                // Date date = new Date(System.currentTimeMillis() + 300 * 1000);
                return new ResponsePage(context, urlName/*, date.getTime()*/);
            }

            context.log("JSP "+ jspFile + " not found, trying HTML", false);

            return getHTMLForCheckOutWorkItem(context);
        } catch (FmcException xcpt) {
            throw new ClientException(context, xcpt);
        }
    }

    /**
    * Creates the page that is displayed after a work item has been
    * force finished. If the work item was actually force finished, the
    * most recent lists is displayed.
    * If it has not yet been force finished because the necessary data
    * has not been specified, the request will be forwarded to
    * <code>forms/ForceFinish.jsp</code> so the parameters for the
    * {@link BuiltinHandler#forceFinishWorkItem forceFinishWorkItem} command
    * can be specified.
    * @param context The request context.
    * @return The response page.
    * @exception ClientException if an error occurred
    */
    public ResponsePage forceFinishWorkItemResponse(RequestContext context)
    throws ClientException
    {
        WorkItem workItem = context.getWorkItem();

        if (context.getRequestParameter("submitData", null) == null)
            return new ResponsePage(context, "/forms/ForceFinish.jsp");

        return mostRecentList(context, !autoRefresh);
    }


    /**
    * Creates the page that is displayed after a work item has been
    * force restarted. If the work item was actually force restarted, the
    * most recent lists is displayed.
    * If it has not yet been force restarted because the necessary data
    * has not been specified, the request will be forwarded to
    * <code>forms/ForceRestart.jsp</code> so the parameters for the
    * {@link BuiltinHandler#forceRestartWorkItem forceRestartWorkItem} command
    * can be specified.
    * @param context The request context.
    * @return The response page.
    * @exception ClientException if an error occurred
    */
    public ResponsePage forceRestartWorkItemResponse(RequestContext context)
    throws ClientException
    {
        WorkItem workItem = context.getWorkItem();

        if (context.getRequestParameter("submitData", null) == null)
            return new ResponsePage(context, "/forms/ForceRestart.jsp");

        return mostRecentList(context, !autoRefresh);

    }

    /**
    * @deprecated - this function will be redesigned in a later release
    */
    public ResponsePage getFilterAssistantResponse(RequestContext context)
    throws ClientException
    {
        return new ResponsePage(context, "/forms/FilterAssistantViewer.jsp");
    }

    /**
    * @deprecated - this function will be redesigned in a later release
    */
    public ResponsePage getSortCriteriaAssistantResponse(RequestContext context)
    throws ClientException
    {
        return new ResponsePage(context, "/forms/SortCriteriaAssistantViewer.jsp");
    }

    /**
    * @deprecated - this function will be redesigned in a later release
    */
    public ResponsePage getUserInfoResponse(RequestContext context)
    throws ClientException
    {
        return new ResponsePage(context, "/forms/UserInfoViewer.jsp");
    }

    /**
    * @deprecated - this function will be redesigned in a later release
    */
    public ResponsePage getSystemInfoResponse(RequestContext context)
    throws ClientException
    {
        return new ResponsePage(context, "/forms/SystemInfoViewer.jsp");
    }


    public Vector createNavigatorEntries(RequestContext context, boolean enableLogoff)
    throws ClientException
    {
        Vector navigatorEntries = new Vector();
        BuiltinHandler builtin = context.getBuiltinHandler();
        String queryParam = autoRefresh ? "" : "&useCache=true";

        try {
            if (enableWorklists) {
                Enumeration worklists = context.getWorkLists();

                if (!worklists.hasMoreElements())
                    worklists = builtin.queryWorkLists(context).getWorkLists();

                WorkList workList = context.getWorkList();
                String worklistName = workList == null ? null : workList.name();

                while (worklists.hasMoreElements()) {
                    WorkList list = (WorkList) worklists.nextElement();
                    String name = list.name();

                    if (name != null && !name.equals(worklistName)) {
                        name = context.get("Navigator.Worklist", name);
                        String command = Command.WL_QUERY_WORK_ITEMS.withParameter(queryParam)
                                         .getURL(context, list.persistentOid(), null, false);
                        navigatorEntries.addElement(new NavigatorEntry(name, command));
                    }
                }
            }

            if (enableTemplatelists) {
                Enumeration templatelists = context.getTemplateLists();

                if (!templatelists.hasMoreElements())
                    templatelists = builtin.queryProcessTemplateLists(context).getTemplateLists();

                ProcessTemplateList templateList = context.getTemplateList();
                String templatelistName = templateList == null ? null : templateList.name();

                while (templatelists.hasMoreElements()) {
                    ProcessTemplateList list = (ProcessTemplateList) templatelists.nextElement();
                    String name = list.name();

                    if (name != null && !name.equals(templatelistName)) {
                        name = context.get("Navigator.Templatelist", name);
                        String command = Command.PTL_QUERY_TEMPLATES.withParameter(queryParam)
                                         .getURL(context, list.persistentOid(), null, false);
                        navigatorEntries.addElement(new NavigatorEntry(name, command));
                    }
                }
            }

            if (enableInstancelists) {
                Enumeration instancelists = context.getInstanceLists();

                if (!instancelists.hasMoreElements())
                    instancelists = builtin.queryProcessInstanceLists(context).getInstanceLists();

                ProcessInstanceList instanceList = context.getInstanceList();
                String instancelistName = instanceList == null ? null : instanceList.name();

                while (instancelists.hasMoreElements()) {
                    ProcessInstanceList list = (ProcessInstanceList) instancelists.nextElement();
                    String name = list.name();

                    if (name != null && !name.equals(instancelistName)) {
                        name = context.get("Navigator.Instancelist", name);
                        String command = Command.PIL_QUERY_INSTANCES.withParameter(queryParam)
                                         .getURL(context, list.persistentOid(), null, false);
                        navigatorEntries.addElement(new NavigatorEntry(name, command));
                    }
                }
            }

            if ((!context.isUsingJNDI())&& enableActivityInstanceLists) {
                Enumeration activityInstanceLists = context.getActivityInstanceLists();

                if (!activityInstanceLists.hasMoreElements())
                    activityInstanceLists = builtin.queryActivityInstanceLists(context).getActivityInstanceLists();

                ActivityInstanceList activityList = context.getActivityInstanceList();
                String activityListName = activityList == null ? null : activityList.name();

                while (activityInstanceLists.hasMoreElements()) {
                    ActivityInstanceList activityInstanceList = (ActivityInstanceList) activityInstanceLists.nextElement();
                    String name = activityInstanceList.name();

                    if (name != null && !name.equals(activityListName)) {
                        name = context.get("Navigator.Activitylist", name);
                        String command = Command.AIL_QUERY_INSTANCES.withParameter(queryParam)
                                         .getURL(context, activityInstanceList.persistentOid(), null, false);
                        navigatorEntries.addElement(new NavigatorEntry(name, command));
                    }
                }
            }

            if ((enableLists) && (context.getInstances() != null || context.getTemplates() != null
                                  || context.getWorkItems() != null || context.getActivities() != null)) {
                navigatorEntries.addElement(new NavigatorEntry(context.get("ListViewer.ShowLists"),
                                                               context.getCommand("queryLists")));
            }
        } catch (FmcException xcpt) {
            throw new ClientException(context, xcpt);
        }

        if (enableLogoff) {
            navigatorEntries.addElement(
                                       new NavigatorEntry(context.get("Navigator.Logoff"),
                                                          context.getCommand("logoff")));
        }

        return navigatorEntries;
    }
    public ResponsePage setUserSettingsResponse(RequestContext context)
    throws ClientException
    {
        return new ResponsePage(context.getRedirect(context.getCommand("getUserInfo")));
    }
    public ResponsePage deleteTemplateResponse(RequestContext context)
    throws ClientException
    {
        return mostRecentList(context, !autoRefresh);
    }
    public ResponsePage cancelWorkItemResponse(RequestContext context)
    throws ClientException
    {
        return mostRecentList(context, !autoRefresh);
    }
    public ResponsePage checkInWorkItemResponse(RequestContext context)
    throws ClientException
    {
        return mostRecentList(context, !autoRefresh);
    }
    public ResponsePage createInstanceResponse(RequestContext context)
    throws ClientException
    {
        return mostRecentList(context, !autoRefresh);
    }
    public ResponsePage createPersistentListResponse(RequestContext context)
    throws ClientException
    {
        PersistentList list = context.getWorkList();

        try {
            if (list != null) {
                context.setWorkListOid(list.persistentOid());
                return mostRecentList(context, false);
            }

            list = context.getInstanceList();

            if (list != null) {
                context.setInstanceListOid(list.persistentOid());
                return mostRecentList(context, false);
            }

            list = context.getTemplateList();

            if (list != null) {
                context.setTemplateListOid(list.persistentOid());
                return mostRecentList(context, false);
            }

            list = context.getActivityInstanceList();
            if (list != null) {
                context.setActivityInstanceListOid(list.persistentOid());
                return mostRecentList(context, false);
            }
        } catch (FmcException xcpt) {
            throw new ClientException(context, xcpt);
        }

        return null;
    }
    public ResponsePage deleteInstanceResponse(RequestContext context) throws ClientException
    {
        return mostRecentList(context, !autoRefresh);
    }
    /**
     * Creates the page that is displayed when an item has been deleted.
     * This page calls {@link BuiltinHandler#queryWorkItems BuiltinHandler.queryWorkItems()}
     * to show the work list again.
     * @param context The request context.
     * @return The response page.
     * @exception ClientException if an error occurred
     */
    public ResponsePage deleteItemResponse(RequestContext context) throws ClientException
    {
        return mostRecentList(context, !autoRefresh);
    }
    /**
            * Creates the page that is displayed when a list has been deleted.
            * This page calls {@link BuiltinHandler#queryLists BuiltinHandler.queryLists()}
            * to show the list of lists again.
            * @param context The request context.
            * @return The response page.
            * @exception ClientException if an error occurred
            */
    public ResponsePage deleteListResponse(RequestContext context)
    throws ClientException
    {
        return mostRecentList(context, !autoRefresh);
    }
    /**
            * Creates the page that is displayed when a work item has been finished.
            * This page calls {@link BuiltinHandler#queryWorkItems BuiltinHandler.queryWorkItems()}
            * to show the work list again.
            * @param context The request context.
            * @return The response page.
            * @exception ClientException if an error occurred
            */
    public ResponsePage finishWorkItemResponse(RequestContext context)
    throws ClientException
    {
        return mostRecentList(context, !autoRefresh);
    }
    /**
            * Creates the page displayed after a process instance has been restarted.
            * This page calls {@link BuiltinHandler#queryProcessInstances BuiltinHandler.queryProcessInstances()}
            * @param context the request context
            * @return The response page.
            * @exception ClientException if an error occurred
            */
    public ResponsePage restartInstanceResponse(RequestContext context)
    throws ClientException
    {
        return mostRecentList(context, !autoRefresh);
    }
    /**
            * Creates the page that is displayed when a work item has been restarted.
            * This page calls {@link BuiltinHandler#queryWorkItems BuiltinHandler.queryWorkItems()}
            * to show the work list again.
            * @param context The request context.
            * @return The response page.
            * @exception ClientException if an error occurred
            */
    public ResponsePage restartWorkItemResponse(RequestContext context)
    throws ClientException
    {
        return mostRecentList(context, !autoRefresh);
    }
    /**
            * Creates the page displayed after a process instance has been resumed.
            * This page calls {@link BuiltinHandler#queryProcessInstances BuiltinHandler.queryProcessInstances()}
            * to show the instance list again.
            * @param context the request context
            * @return The response page.
            * @exception ClientException if an error occurred
            */
    public ResponsePage resumeInstanceResponse(RequestContext context)
    throws ClientException
    {
        return mostRecentList(context, !autoRefresh);
    }
    /**
            * @deprecated - this function will be redesigned in a later release
            */
    public ResponsePage setViewSettingsResponse(RequestContext context)
    throws ClientException
    {
        return new ResponsePage(context.getRedirect(context.getCommand("showSettings")));
    }
    /**
            * Creates the page that is displayed when a work item has been started.
            * This page calls {@link BuiltinHandler#queryWorkItems BuiltinHandler.queryWorkItems()}
            * to show the work list again.
            * @param context The request context.
            * @return The response page.
            * @exception ClientException if an error occurred
            */
    public ResponsePage startWorkItemResponse(RequestContext context)
    throws ClientException
    {
        return mostRecentList(context, !autoRefresh);
    }
    /**
            * Creates the page that is displayed when a work item has been terminated.
            * This page calls {@link BuiltinHandler#queryWorkItems BuiltinHandler.queryWorkItems()}
            * to show the work list again.
            * @param context The request context.
            * @return The response page.
            * @exception ClientException if an error occurred
            */
    public ResponsePage terminateWorkItemResponse(RequestContext context)
    throws ClientException
    {
        return mostRecentList(context, !autoRefresh);
    }
    /**
            * Creates the page displayed after a process instance has been terminated.
            * This page calls {@link BuiltinHandler#queryProcessInstances BuiltinHandler.queryProcessInstances()}
            * to show the instance list again.
            * @param context the request context
            * @return The response page.
            * @exception ClientException if an error occurred
            */
    public ResponsePage terminateInstanceResponse(RequestContext context)
    throws ClientException
    {
        return mostRecentList(context, !autoRefresh);
    }
    /**
     * Creates the page that is displayed when an item has been transferred.
     * If the item was transferred, the work list will be shown again.
     * If the item was not yet transferred because the user ID to transfer
     * to was not specified, the HTML template file <code>forms/Transfer.html</code>
     * will be loaded which may contain the following substitution variables:
     * <table border="1" cellspacing="0" cellpadding="4">
     *   <tr>
     *     <th>Name</th>
     *     <th>Replacement</th>
     *   </tr>
     *   <tr>
     *     <td>{_HTMLDir_}</td>
     *     <td>The root URI of the servlet's web application, for example,
     *         <code>/MQWFClient</code>.</td>
     *   </tr>
     *   <tr>
     *     <td>{_MQWFServlet_}</td>
     *     <td>The servlet's name, for example,
     *         <code>/MQWFClient/servlet/Main</code>.</td>
     *   </tr>
     *   <tr>
     *     <td>{_Oid_}</td>
     *     <td>The OID of the item that is to be transferred.</td>
     *   </tr>
     *   <tr>
     *     <td>{_ItemName_}</td>
     *     <td>The name of the item that is to be transferred.</td>
     *   </tr>
     * </table>
     * @param context The request context.
     * @return The response page.
     * @exception ClientException if an error occurred
     */
    public ResponsePage transferItemResponse(RequestContext context)
    throws ClientException
    {
        Item item = context.getItem();

        if (context.getRequestParameter("userID", null) != null) {
            // transfer successful
            return mostRecentList(context, !autoRefresh);
        } else {
            // userID needs to be supplied
            Properties subst = new Properties();

            try {
                subst.put("{_ItemName_}", item.name());
                subst.put("{_Oid_}", item.persistentOid());
            } catch (FmcException xcpt) {
                throw new ClientException(context, xcpt);
            }

            String urlName = context.getURL("{_HTMLDir_}/forms/Transfer.html");

            try {
                StringBuffer answer = context.loadTemplateFile(urlName, subst);
                return new ResponsePage(answer);
            } catch (IOException xcpt) {
                context.log("Template file " + urlName + " could not be read: " + Util.toString(xcpt), true);
                throw new ClientException(context, context.get("Error.HTMLTemplateNotFound", urlName));
            }
        }
    }
    protected ResponsePage redirectResponse(RequestContext context, String urlname,
                                            Command command, String id, String addl)
    {
        String commandString = command.getCommand();

        if (commandString.equals(context.getRequest().getParameter("command"))) {
            if (urlname == null)
                urlname = context.getCommand(commandString, id, addl);

            return new ResponsePage(context, urlname);
        }

        if (commandString.startsWith("query")) {
            if (addl == null)
                addl = "";

            addl = addl + "&useCache=true";
        }

        return new ResponsePage((RequestContext) null, context.getCommand(commandString, id, addl));
    }
    protected ResponsePage mostRecentList(RequestContext context,
                                          boolean useCache)
    throws ClientException
    {
        int kindOfList = context.mostRecentList(enableWorklists, enableInstancelists,
                                                enableTemplatelists, enableLists, useCache);

        switch (kindOfList) {
        case SessionContext.TEMPLATELIST:
            return queryProcessTemplatesResponse(context);

        case SessionContext.INSTANCELIST:
            return queryProcessInstancesResponse(context);

        case SessionContext.WORKLIST:
            return queryWorkItemsResponse(context);

        case SessionContext.LISTS:
            return queryListsResponse(context);

        case SessionContext.ACTIVITYINSTANCELIST:
            return queryActivityInstancesResponse(context);
        }

        return null;
    }

    private ResponsePage getHTMLTemplateForCreateAndStart(RequestContext context)
    throws ClientException
    {
        ResponsePage page = null;

        try {
            ProcessInstance instance = context.getInstance();
            String templateName = context.getTemplate().name();
            String templatelistOid = context.getTemplateListOid();
            String oid = context.getRequestParameter("id", null);

            if (instance == null) {
                // not yet started because input data required
                String urlname = context.getURL("{_HTMLDir_}/processes/" + templateName + ".html");

                StringBuffer actionArea = new StringBuffer(1024);
                actionArea.append("<input type=\"hidden\" name=\"command\"")
                .append(" value=\"")
                .append(Command.PT_CREATE_AND_START_INSTANCE.getCommand())
                .append("\">\r\n");
                actionArea.append("<input type=\"hidden\" name=\"id\"")
                .append(" value=\"").append(oid)
                .append("\">\r\n");
                actionArea.append("<input type=\"submit\" name=\"submitData\"")
                .append(" value=\"")
                .append(context.get("Command.StartInstance")).append("\">\r\n");
                actionArea.append("<input type=\"button\" name=\"cancel\"")
                .append(" value=\"").append(context.get("Command.Cancel"))
                .append("\" onClick=\"javascript:location.replace('")
                .append(Command.PTL_QUERY_TEMPLATES.withParameter("&useCache=true")
                        .getURL(context, templatelistOid, null, false))
                .append("')\">\r\n");

                Properties subst = new Properties();
                subst.put("{_ActionArea_}", actionArea.toString());

                try {
                    StringBuffer answer = context.loadTemplateFile(urlname, subst);
                    page = new ResponsePage(answer);
                } catch (IOException xcpt) {
                    if (!generateForms)
                        throw new ClientException(context, context.get("Error.HTMLTemplateNotFound", urlname));

                    context.log("Template file " + urlname + " could not be read: " + Util.toString(xcpt), false);

                    ReadWriteContainer inData = context.getTemplate().initialInContainer();

                    StringBuffer answer = context.openPage(context.get("Viewer.CreateInstanceTitle"), 2048);
                    answer.append("<body>\r\n");

                    // Create a table that holds all input fields
                    answer.append("<h2>")
                    .append(context.get("Viewer.CreateInstanceHeader", templateName))
                    .append("</h2>\r\n");
                    answer.append("<br>\r\n");

                    answer.append(context.generateForm(Command.PT_CREATE_AND_START_INSTANCE, oid,
                                                       Command.PTL_QUERY_TEMPLATES.withText("Command.Cancel")
                                                       .withParameter("&useCache=true"),
                                                       templatelistOid, null, inData, templateName, null));

                    answer.append("</body></html>");

                    return new ResponsePage(answer);
                }
            } else {
                // instance != null - process successfully started
                String urlname = null;
                String useRedirect = null;

                if (oid != null) {
                    // process was started from template list
                    urlname = Command.PTL_QUERY_TEMPLATES.withParameter("&useCache=true")
                              .getURL(context, templatelistOid, null, false);
                    useRedirect = Boolean.TRUE.toString();
                } else {
                    // process was started by the webuser
                    urlname = context.getRequestParameter("SuccessLocation",
                                                          "{_HTMLDir_}/forms/ProcessStarted.html");
                    urlname = context.getURL(urlname);
                    useRedirect = context.getRequestParameter("UseRedirect", Boolean.FALSE.toString());
                }

                if (Util.booleanValue(useRedirect, false)) {
                    page = new ResponsePage(context.getRedirect(urlname));
                } else {
                    Properties subst = new Properties();
                    subst.put("{_InstanceName_}", instance.name());
                    subst.put("{_TemplateName_}", templateName);

                    try {
                        StringBuffer answer = context.loadTemplateFile(urlname, subst);
                        page = new ResponsePage(answer);
                    } catch (IOException xcpt) {
                        context.log("Template file " + urlname + " could not be read: " + Util.toString(xcpt), true);
                        throw new ClientException(context, context.get("Error.HTMLTemplateNotFound", urlname));
                    }
                }
            }
        } catch (FmcException xcpt) {
            throw new ClientException(context, xcpt);
        }

        return page;
    }
    private ResponsePage getStartInstanceResponseHTMLfile(RequestContext context)
    throws ClientException
    {
        ProcessInstance instance = context.getInstance();
        ResponsePage page = null;

        try {
            if (instance.state().value() !=
                com.ibm.workflow.api.ProcessInstancePackage.ExecutionState._READY) {
                page = mostRecentList(context, !autoRefresh);
            } else {
                // instance.state() == READY - not yet started because input data required
                String templateName = instance.processTemplateName();
                String urlname = context.getURL("{_HTMLDir_}/processes/" + templateName + ".html");
                String instancelistOid = context.getInstanceListOid();
                String oid = instance.persistentOid();

                StringBuffer actionArea = new StringBuffer(1024);
                actionArea.append("<input type=\"hidden\" name=\"command\"")
                .append(" value=\"")
                .append(Command.PI_START.getCommand())
                .append("\">\r\n");
                actionArea.append("<input type=\"hidden\" name=\"id\"")
                .append(" value=\"").append(oid)
                .append("\">\r\n");
                actionArea.append("<input type=\"submit\" name=\"submitData\"")
                .append(" value=\"")
                .append(context.get("Command.StartInstance"))
                .append("\">\r\n");
                actionArea.append("<input type=\"button\" name=\"cancel\"")
                .append(" value=\"").append(context.get("Command.Cancel"))
                .append("\" onClick=\"javascript:location.replace('")
                .append(Command.PIL_QUERY_INSTANCES.withParameter("&useCache=true")
                        .getURL(context, instancelistOid, null, false))
                .append("')\">\r\n");

                Properties subst = new Properties();
                subst.put("{_ActionArea_}", actionArea.toString());

                try {
                    StringBuffer answer = context.loadTemplateFile(urlname, subst);
                    page = new ResponsePage(answer);
                } catch (IOException xcpt) {
                    if (!generateForms)
                        throw new ClientException(context, context.get("Error.HTMLTemplateNotFound", urlname));

                    context.log("Template file " + urlname + " could not be read: " + Util.toString(xcpt), false);

                    ReadWriteContainer inData = instance.inContainer();

                    StringBuffer answer = context.openPage(context.get("Viewer.StartInstanceTitle"), 2048);
                    answer.append("<body>\r\n");

                    // Create a table that holds all input fields
                    answer.append("<h2>")
                    .append(context.get("Viewer.StartInstanceHeader", instance.name()))
                    .append("</h2>\r\n");
                    answer.append("<br><br>\r\n");

                    answer.append(context.generateForm(Command.PI_START, oid,
                                                       Command.PIL_QUERY_INSTANCES.withText("Command.Cancel")
                                                       .withParameter("&useCache=true"),
                                                       instancelistOid, null, inData, instance.name(), null));

                    answer.append("</body></html>");

                    return new ResponsePage(answer);
                }
            }
        } catch (FmcException xcpt) {
            throw new ClientException(context, xcpt);
        }

        return page;
    }
    private ResponsePage getHTMLForCheckOutWorkItem(RequestContext context)
    throws ClientException
    {
        WorkItem workItem = context.getWorkItem();
        ReadOnlyContainer inData = context.getContainer();

        try {
            if (inData == null) {
                // work item has been started, not checked out
                return mostRecentList(context, !autoRefresh);
            } else {
                String workItemName = workItem.name();
                String workItemOid = workItem.persistentOid();
                Properties subst = new Properties();

                Date now = new Date();
                subst.put("{_Date_}", context.formatDate(now));
                subst.put("{_Time_}", context.formatTime(now));
                subst.put("{_Name_}", workItem.name());
                subst.put("{_Description_}", null2Empty(workItem.description()));
                subst.put("{_UserID_}", context.getUserID());

                StringBuffer actionArea = new StringBuffer(1024);
                actionArea.append("<input type=\"hidden\" name=\"command\"")
                .append(" value=\"").append(Command.WI_CHECKIN.getCommand())
                .append("\">\r\n");
                actionArea.append("<input type=\"hidden\" name=\"id\"")
                .append(" value=\"").append(workItemOid)
                .append("\">\r\n");
                actionArea.append("<input type=\"submit\" name=\"submitData\"")
                .append(" value=\"")
                .append(context.get("Viewer.CheckInWorkItem"))
                .append("\">\r\n");
                actionArea.append("<input type=\"button\" name=\"cancel\"")
                .append(" value=\"").append(context.get("Command.Cancel"))
                .append("\" onClick=\"javascript:location.replace('")
                .append(Command.WI_CANCEL_CHECKOUT
                        .getURL(context, workItemOid, null, false))
                .append("')\">\r\n");

                subst.put("{_ActionArea_}", actionArea.toString());

                String urlName = "{_HTMLDir_}/programs/" + workItem.implementation() + ".html";
                urlName = context.getURL(urlName);
                ResponsePage page = null;

                try {
                    StringBuffer answer = context.loadTemplateFile(urlName, subst);
                    answer = replaceINDataValues(context, answer.toString(), inData);
                    page = new ResponsePage(answer);
                } catch (IOException xcpt) {
                    if (!generateForms)
                        throw new ClientException(context, context.get("Error.HTMLTemplateNotFound", urlName));

                    context.log("Template file " + urlName + " could not be read: " + Util.toString(xcpt), false);

                    StringBuffer answer = context.openPage(context.get("Viewer.CheckInWorkItemTitle"), 2048);
                    answer.append("<body>\r\n");

                    // Create a table that holds all input fields
                    answer.append("<h2>")
                    .append(context.get("Viewer.CheckInWorkItemHeader", workItemName))
                    .append("</h2>\r\n");
                    answer.append("<br><br>\r\n");

                    answer.append(context.generateForm(Command.WI_CHECKIN, workItemOid,
                                                       Command.WI_CANCEL_CHECKOUT, workItemOid,
                                                       inData, workItem.outContainer(), null, "0"));

                    answer.append("</body></html>");

                    page = new ResponsePage(answer);
                }

                return page;
            }
        } catch (FmcException xcpt) {
            throw new ClientException(context, xcpt);
        }
    }
    /**
            * @deprecated - this function will be redesigned in a later release
            */
    protected final static String null2Empty(String str)
    {
        return Util.null2Default(str, "");
    }
    /**
            * @deprecated - this function will be redesigned in a later release
            */
    protected StringBuffer replaceINDataValues(RequestContext context,
                                               String contents,
                                               ReadOnlyContainer inData)
    {
        StringBuffer newContents = new StringBuffer (1000);
        int beginPos = 0;
        int endPos = 0;
        int idx = 0;

        // Replace all input fields with input data values
        while ((beginPos = contents.indexOf("{IN.", endPos)) > 0) {
            // Replace the data field with the actual value from input container
            endPos = contents.indexOf('}', beginPos);
            // Get the input item name
            String itemName = contents.substring(beginPos + 4, endPos);
            // Get corresponding output container element
            newContents.append(contents.substring(idx, beginPos));
            try {
                String itemValue = context.getMemberValue(inData, itemName, "&nbsp;");
                // Replace with item value
                newContents.append(itemValue);
            } catch (FmcException xcpt) {
                // If there is a problem, replace the placeholder with the error message
                newContents.append(xcpt.getMessage());
            }

            idx = endPos + 1;
        } // end of while

        // Append rest of the buffer to the new buffer
        newContents.append(contents.substring(idx));

        return newContents;
    }
    /**
            * @deprecated - this function will be redesigned in a later release
            */
    public static void getContainerData(RequestContext context, StringBuffer answer,
                                        String contType, String structName,
                                        ContainerElement[] elements,
                                        Container compData)
    {
        try {
            for (int i = 0; elements != null && i < elements.length; ++i) {
                ContainerElement element = elements[i];
                //String itemname = element.name(); -> use fullName to automatically receive the qualified names
                String itemname = element.fullName();

                // build qualified item name
                //if (structName.length() > 0)
                //   itemname = structName + "." + itemname;

                if (!element.isArray())
                    getElementData(context, answer, contType, itemname, element, compData);
                else {
                    ContainerElement[] elems = element.arrayElements();

                    for (int j = 0; elems != null && j < elems.length; ++j) {
                        //getElementData(context, answer, contType,
                        //               itemname + "[" + j + "]", elems[j], compData);
                        getElementData(context,answer,contType,elems[j].fullName(),elems[j],compData);
                    }
                }
            }
        } catch (FmcException ignored) {
        }
    }
    /**
            * @deprecated - this function will be redesigned in a later release
            */
    private static void getElementData(RequestContext context, StringBuffer answer, String contType,
                                       String itemname, ContainerElement element,
                                       Container compData)
    {
        try {
            if (element.isStruct()) {
                context.log("Element " + itemname + " is struct -> calling getContainerData recursively", false);
                // recursive call to process all structure elements
                getContainerData(context, answer, contType, itemname,
                                 element.structMembers(), compData);
            } else {
                if (contType.equals("OUT")) {
                    boolean existsInInputContainer = false;

                    try {
                        if (compData != null)
                            existsInInputContainer = compData.getElement(itemname) != null;
                    } catch (FmcException ignored) {
                    } finally {
                        if (!existsInInputContainer) {
                            answer.append("<tr>");
                            answer.append("<td>").append(itemname).append("</td>");
                            answer.append("<td>").append(element.type()).append("</td>");
                            answer.append("<td>");

                            if ("BINARY".equals(element.type())) {
                                answer.append(context.getMemberValue(element, ""));
                            } else {
                                answer.append("<input type=\"text\" name=\"")
                                .append(itemname).append("\"")
                                .append(" value=\"")
                                .append(context.getMemberValue(element, ""));

                                if (element.type().equals("STRING"))
                                    answer.append("\" size=\"60\">");
                                else
                                    answer.append("\" size=\"15\">");
                            }

                            answer.append("</td></tr>\r\n");
                        }
                    }
                } else {
                    // contType == "IN"
                    answer.append("<tr>");
                    answer.append("<td>").append(itemname).append("</td>");
                    answer.append("<td>").append(element.type()).append("</td>");
                    String itemvalue = context.getMemberValue(element, null);
                    boolean written = false;

                    // if this item exists in the output container then create a field else
                    // just write text value
                    try {
                        // @EF what if types are different? provide default mapping anyway?
                        String compvalue = null;

                        try {
                            if (compData != null) {
                                ContainerElement from = compData.getElement(itemname);

                                if (from != null)
                                    compvalue = context.getMemberValue(from, "");
                            }
                        } catch (FmcException ignored) {
                        }

                        if (compvalue != null && !"BINARY".equals(element.type())) {
                            answer.append("<td><input type=\"text\" name=\"")
                            .append(itemname).append("\"")
                            .append(" value=\"")
                            .append(RequestContext.toHTMLText(itemvalue, compvalue));

                            if (element.type().equals("STRING"))
                                answer.append("\" size=\"60\"></td>");
                            else
                                answer.append("\" size=\"15\"></td>");

                            written = true;
                        }
                    } catch (FmcException ignored) {
                    } finally {
                        if (!written) {
                            answer.append("<td>")
                            .append(RequestContext.toHTMLText(itemvalue, "&nbsp;"))
                            .append("</td>");
                        }
                    }
                    answer.append("</tr>\r\n");
                }
            }
        } catch (FmcException ignored) {
        }
    }
    protected boolean enableTemplatelists = true;
    protected boolean enableInstancelists = true;
    protected boolean enableWorklists = true;
    protected boolean enableLists = true;
    protected boolean enableActivityInstanceLists=true;

    private String  pageAfterLogoff;
    private boolean autoRefresh = false;
    private boolean generateForms = true;

}
