/*BEGIN CMVC */
//*************************************************************************
//
//  Workfile: servlet/sample/HTMLDocHandler.java, v3-java, fmv3
//  Last update: 01/01/29 15:32:09
//  SCCS path, id: /home/flowmark/vc/0/3/5/5/s.34 1.6
//
//************************************************************************/
/*END CMVC */
/*BEGIN COPYRIGHT */
//*****************************************************************************
//
//  Licensed Materials - Property of IBM
//  5697-FM3 5655-A96
//  (C) Copyright IBM Corp. 1999, 2008
//  US Government Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract
//  with IBM Corp.
//
//****************************************************************************/
//END COPYRIGHT */
/*BEGIN DISCLAIMER */
//*************************************************************************
//  DISCLAIMER
//  ----------
//  This material contains programming source code for your consideration.
//  These examples have not been thoroughly tested under all conditions.
//  IBM, therefore, cannot guarantee or imply reliability, serviceability,
//  or function of these programs.
//  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
//  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
//  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
//  FOR A PARTICULAR PURPOSE.
//************************************************************************/
/*END DISCLAIMER */

package com.ibm.workflow.servlet.sample;

import java.io.*;
import com.ibm.workflow.api.*;
import com.ibm.workflow.servlet.client.*;

/**
* <p>This class provides a link from process template lists to the HTML
* documentation exported from Buildtime. It does not define a new command but
* simply creates a hyperlinked icon for processes where online documentation is
* available. It is assumed that the HTML documentation is located in the
* <code>ProcessDocs</code> subdirectory of the Web Client's document root
* directory.</p>
* <p>For details on how to run this sample, see the
* <a href="../../../../../../samples.html#htmldoc">Samples</a> section.</p>
*/
public class HTMLDocHandler extends GenericCommandHandler
{
  /**
  * Returns information about the handler, such as author, version,
  * and copyright.
  * @return a String containing handler information
  */
  public String getHandlerInfo()
  {
    return "MQWF Sample - Show Process Documentation V1.1";
  }

  /**
  * Returns an HTML tag to be included in process template lists. This tag points
  * to the online process documentation where available.
  * @return a String containing a <code>href</code> or an empty String if
  *         no online documentation is available
  */
  public String getTriggerTagFor(int list, RequestContext context)
  {
    String tag = "";

    // show icon only on template lists
    if (list == SessionContext.TEMPLATELIST)
    {
      try
      {
        // the name of the HTML file containing the process documentation
        // exported from Buildtime
        String name = "/ProcessDocs/" + context.getTemplate().name() + ".htm";

        // show icon only if the HTML documentation exists
        if (new File(context.getDocumentRoot(), name).isFile())
        {
          tag = "<a href='javascript:{}' onClick=\"javascript:window.open('"
              + context.getURL("{_HTMLDir_}" + name)
              + "','Process_Documentation','scrollbars,resizable')\">"
              + cmd.getActionIcon(context, "border=\"0\"") + "</a>";
        }
      }
      catch (FmcException xcpt)
      {
        getContext().log("Querying process template name: ", xcpt);
      }
    }

    return tag;
  }

  private static final Command cmd = new Command(null, "htmldoc.gif",
                                                 "Process Documentation");
}
