/*BEGIN CMVC */
//*************************************************************************
//
//  Workfile: servlet/sample/InternetConnectionViewer.java, v3-java, fmv3
//  Last update: 01/05/07 16:45:40
//  SCCS path, id: /home/flowmark/vc/0/3/5/5/s.35 1.7
//
//************************************************************************/
/*END CMVC */
/*BEGIN COPYRIGHT */
//*****************************************************************************
//
//  Licensed Materials - Property of IBM
//  5697-FM3 5655-A96
//  (C) Copyright IBM Corp. 1999, 2008
//  US Government Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract
//  with IBM Corp.
//
//****************************************************************************/
//END COPYRIGHT */
/*BEGIN DISCLAIMER */
//*************************************************************************
//  DISCLAIMER
//  ----------
//  This material contains programming source code for your consideration.
//  These examples have not been thoroughly tested under all conditions.
//  IBM, therefore, cannot guarantee or imply reliability, serviceability,
//  or function of these programs.
//  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
//  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
//  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
//  FOR A PARTICULAR PURPOSE.
//************************************************************************/
/*END DISCLAIMER */

package com.ibm.workflow.servlet.sample;

import java.util.Enumeration;
import java.text.MessageFormat;
import javax.servlet.http.HttpSession;

import com.ibm.workflow.api.*;
import com.ibm.workflow.api.ItemPackage.*;
import com.ibm.workflow.servlet.client.*;

/**
* <p>This Viewer mimics the look and feel of the Internet Connection for
* IBM FlowMark V2. It does so by displaying a custom work list after
* a successful logon request. Note that this Viewer extends the DefaultViewer
* class and only overrides two methods instead of implementing the full
* Viewer interface.</p>
* <p>This sample also demonstrates how to use the {@link Catalog} class. For
* demonstration purposes, the German version of the messages used in this class
* has been included. Just switch your browser's language to German...</p>
* <p>For details on how to run this sample, see the
* <a href="../../../../../../samples.html#icviewer">Samples</a> section.</p>
*/
public class InternetConnectionViewer extends DefaultViewer
{
  /**
  * This method is called to create the response page for the
  * {@link BuiltinHandler#logon BuiltinHandler.logon()} command. It will
  * display the items of the first work list it finds.
  * @param context The request context.
  * @return The response page.
  * @exception com.ibm.workflow.servlet.client.ClientException If no work list is available.
  */
  public ResponsePage logonResponse(RequestContext context)
                      throws ClientException
  {
    // create the message catalog for this client
    Catalog cat = context.getCatalog("com.ibm.workflow.catalog.servlet.sample.Sample");
    HttpSession session = context.getRequest().getSession(false);
    session.putValue(CATALOG_KEY, cat); // @V22: setAttribute()

    // fill the SessionContext cache with the available work lists
    BuiltinHandler builtin = context.getBuiltinHandler();
    builtin.queryWorkLists(context);

    // query the first work list
    Enumeration workLists = context.getWorkLists();

    WorkList workList = workLists.hasMoreElements()
                      ? (WorkList) workLists.nextElement()
                      : null;

    if (workList == null)
      throw new ClientException(context, cat.get("Error.NoLists"));

    // query the work items on the work list
    try
    {
      String id = workList.persistentOid();
      boolean useCache = false;

      return queryWorkItemsResponse(builtin.queryWorkItems(context, id, useCache));
    }
    catch (FmcException xcpt)
    {
      throw new ClientException(context, xcpt);
    }
  }

  /**
  * This method is called to create the response page for the
  * {@link BuiltinHandler#queryWorkItems BuiltinHandler.queryWorkItems()} command.
  * @param context The request context.
  * @return The response page.
  * @exception com.ibm.workflow.servlet.client.ClientException If an error occurred.
  */
  public ResponsePage queryWorkItemsResponse(RequestContext context)
                      throws ClientException
  {
    // retrieve the message catalog for this client
    HttpSession session = context.getRequest().getSession(false);
    Catalog cat = (Catalog) session.getValue(CATALOG_KEY); // @V22: getAttribute()

    try
    {
      // create html tags for the images used
      String headImage    = "<img src=\"" +
        context.getURL("{_HTMLDir_}/images/workflow.gif") +
        "\" alt=\"" + cat.get("InternetConnectionList.Header") + "\">";

      String actionImageAttr = "align=\"texttop\" border=\"0\"";

      String startImage   = "<img src=\"" +
        context.getURL("{_HTMLDir_}/images/point.gif") +
        "\" " + actionImageAttr + " alt=\"" +
        cat.get("InternetConnectionList.Start") + "\">";

      String cancelCheckoutImage = "<img src=\"" +
        context.getURL("{_HTMLDir_}/images/restart.gif") +
        "\" " + actionImageAttr + " alt=\"" +
        cat.get("InternetConnectionList.CancelCheckout") + "\">";

      String restartImage   = "<img src=\"" +
        context.getURL("{_HTMLDir_}/images/restart.gif") +
        "\" " + actionImageAttr + " alt=\"" +
        cat.get("InternetConnectionList.Restart") + "\">";

      // create the work list table header
      StringBuffer answer = new StringBuffer(10000);
      answer.append("<html><head><title>")
      .append(cat.get("InternetConnectionResponse.Worklist"))
            .append("</title></head><body>\r\n");

      answer.append("<font size=3>\r\n<br>\r\n")
            .append(headImage)
            .append("\r\n<h4>")
            .append(cat.get("InternetConnectionList.Select"))
            .append("</h4>\r\n")
            .append("<table width=\"100%\" border=\"1\">\r\n")
            .append("  <tr bgcolor=\"#CFDDFE\">\r\n")
            .append("    <th width=\"65%\">")
            .append(cat.get("InternetConnectionList.Description"))
            .append("</th>\r\n")
            .append("    <th width=\"15%\">")
            .append(cat.get("InternetConnectionList.Status"))
            .append("</th>\r\n")
            .append("    <th width=\"20%\">")
            .append(cat.get("InternetConnectionList.Received"))
            .append("</th>\r\n")
            .append("  </tr>\r\n");

      // create a line for each work on the list, using only applicable
      // actions depending on the work item state
      for (int i = 0; i < context.getWorkItems().length; ++i)
      {
        try
        {
          WorkItem workItem  = context.getWorkItems()[i];
          workItem.refresh();
          String oid = workItem.persistentOid();
          String description = workItem.description();

          answer.append("  <tr valign=\"top\">\r\n")
                .append("  <td valign=\"top\" wrap>\r\n")
                .append("    <table cellspacing=\"2\" cellpadding=\"2\">\r\n")
                .append("      <tr>\r\n")
                .append("        <td>\r\n")
                .append("          <a href=\"");

          if (workItem.state() == ExecutionState.READY)
          {
            answer.append(context.getCommand("checkOutWorkItem", oid))
                  .append("\">\r\n            ")
                  .append(MessageFormat.format(startImage,
                                               new String[] { description }));
          }
          else if (workItem.state() == ExecutionState.CHECKED_OUT)
          {
            answer.append(context.getCommand("cancelWorkItem", oid))
                  .append("\">\r\n")
                  .append(MessageFormat.format(cancelCheckoutImage,
                                               new String[] { description }));
          }
          else
          {
            answer.append(context.getCommand("forceRestartWorkItem", oid))
                  .append("\">\r\n")
                  .append(MessageFormat.format(restartImage,
                                               new String[] { description }));
          }

          answer.append("</a>\r\n        </td>\r\n")
                .append("        <td align=\"left\" valign=\"top\">")
                .append(description).append("</td>\r\n")
                .append("      </tr>\r\n")
                .append("    </table>\r\n")
                .append("  </td>\r\n")
                .append("  <td align=\"center\" valign=\"top\">")
                .append(context.getWIStringForState(workItem.state()))
                .append("</td>\r\n")
                .append("  <td align=\"center\" valign=\"top\">")
                .append(context.toString(workItem.creationTime()))
                .append("</td>\r\n")
                .append("  </tr>\r\n");
        }
        catch(FmcException xcpt)
        {
          if (xcpt.rc != xcpt.FMC_ERROR_DOES_NOT_EXIST)
            throw xcpt;
        }
      }

      // create the work list table footer with refresh and logoff buttons
      answer.append("</table>\r\n")
            .append("<hr>\r\n")
            .append(context.openForm("queryWorkItems", context.getWorkList().persistentOid()))
            .append("<input type=\"submit\" value=\"")
            .append(cat.get("InternetConnectionList.Refresh"))
            .append("\" name=\"Refresh\">\r\n")
            .append("</form>\r\n")
            .append("<hr>\r\n")
            .append(context.openForm("logoff"))
            .append("<input type=\"submit\" value=\"")
            .append(cat.get("InternetConnectionList.Logoff"))
            .append("\" name=\"Logoff\">\r\n")
            .append("</form></font>\r\n");
      answer.append("</body></html>");

      return new ResponsePage(answer);
    }
    catch (FmcException xcpt)
    {
      throw new ClientException(context, xcpt);
    }
  }

  private static final String CATALOG_KEY = "InternetConnectionViewer.Catalog";
}
