/*BEGIN CMVC */
//*************************************************************************
//
//  Workfile: servlet/sample/StarterPEAHandler.java, v3-java, fmv3
//  Last update: 01/02/12 13:58:43
//  SCCS path, id: /home/flowmark/vc/0/3/5/5/s.37 1.6
//
//************************************************************************/
/*END CMVC */
/*BEGIN COPYRIGHT */
//*****************************************************************************
//
//  Licensed Materials - Property of IBM
//  5697-FM3 5655-A96
//  (C) Copyright IBM Corp. 1999, 2008
//  US Government Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract
//  with IBM Corp.
//
//****************************************************************************/
//END COPYRIGHT */
/*BEGIN DISCLAIMER */
//*************************************************************************
//  DISCLAIMER
//  ----------
//  This material contains programming source code for your consideration.
//  These examples have not been thoroughly tested under all conditions.
//  IBM, therefore, cannot guarantee or imply reliability, serviceability,
//  or function of these programs.
//  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
//  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
//  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
//  FOR A PARTICULAR PURPOSE.
//************************************************************************/
/*END DISCLAIMER */

package com.ibm.workflow.servlet.sample;

import javax.servlet.http.*;

import com.ibm.workflow.api.*;
import com.ibm.workflow.servlet.client.*;

/**
* <p>This CommandHandler automatically starts and stops the Program Execution
* Agent for the Web user.</p>
* <p>This sample demonstrates how to use the
* {@link CommandHandler#onLogon(RequestContext) onLogon}
* and {@link CommandHandler#onLogoff onLogoff} hooks.</p>
* <p>For details on how to run this sample, see the
* <a href="../../../../../../samples.html#pea">Samples</a> section.</p>
*/
public class StarterPEAHandler extends GenericCommandHandler
{
  /**
  * Returns information about the handler, such as author, version,
  * and copyright.
  * @return a String containing handler information
  */
  public String getHandlerInfo()
  {
    return "MQWF Sample - Start PEA for Web user V1.1";
  }

  /**
  * Initializes the handler and caches the Web user, if configured. This
  * feature has to be enabled in
  * <a href="../../../../../../properties.html#starter">WebClient.properties</a>
  * by setting <code>StarterUserID</code> and <code>StarterPEA</code>.
  * @param config  The servlet's configuration and initialization parameters
  */
  public void init(Config config)
  {
    super.init(config);

    starterUserID = config.getParameter("Servlet", "StarterUserID");
    if (starterUserID != null) starterUserID.toUpperCase();
    starterHasPEA = config.getParameter("Servlet", "StarterPEA", false);
  }

  /**
  * Start the Web user's PEA called for the Web user ID and if enabled in
  * <code>WebClient.properties</code>. If the PEA already is running, it
  * will be receycled.
  * @param context The {@link RequestContext} of the newly added session
  */
  public void onLogon(RequestContext context)
  {
    String userID = context.getUserID();

    if (starterHasPEA && userID.equals(starterUserID))
      startStarterPEA(context.getExecutionService(), userID, true);
  }

  /**
  * Shuts down the Web user's PEA if called for the Web user ID. Note that
  * this will only happen if you terminate your Web application properly.
  * There is no way to explicitely log off the Web user.
  * @param context The {@link SessionContext} of the session to be removed
  */
  public void onLogoff(SessionContext context)
  {
    String userID = context.getUserID();

    if (starterHasPEA && userID.equals(starterUserID))
      shutDownStarterPEA(context.getExecutionService(), userID);
  }

  /**
  * Starts the PEA for the given ExecutionService
  * @param service the ExcecutionService to start the PEA for
  * @param userID used for logging only
  * @param restart if <code>true</code>, the PEA will be receycled if it
  *                is already running.
  */
  private void startStarterPEA(ExecutionService service,
                               String userID, boolean restart)
  {
    try
    {
      service.programExecutionAgentStartUp();
      getContext().log("  Started PEA for " + userID + ".", false);
    }
    catch (FmcException xcpt)
    {
      if (xcpt.rc != FmcException.FMC_ERROR_ALREADY_STARTED || !restart)
        getContext().log("Starting PEA for " + userID, xcpt);
      else
      {
        getContext().log("  PEA already running, restarting.", false);
        shutDownStarterPEA(service, userID);
        startStarterPEA(service, userID, false);
      }
    }
  }

  /**
  * Stops the PEA for the given ExecutionService
  * @param service the ExcecutionService to start the PEA for
  * @param userID used for logging only
  */
  private void shutDownStarterPEA(ExecutionService service, String userID)
  {
    try
    {
      service.programExecutionAgentShutDown();
      getContext().log("  Shut down PEA for " + userID + ".", false);
    }
    catch (FmcException xcpt)
    {
      getContext().log("  Error shutting down PEA", xcpt);
    }
  }

  private String  starterUserID;
  private boolean starterHasPEA = false;
}

