import java.io.*;
import com.ibm.workflow.api.*;
import com.ibm.workflow.api.ServicePackage.*;

public class Group
{
  private static PrintWriter out = new PrintWriter(System.out, true);

  public static void main (String[] args)
  {
    try
    {
      if (args.length == 0)
      {
        out.println("Usage: Group <option>");
        out.println("Where <option> is one of the following:");
        out.println("  dwl - delete the work lists created by GroupWorkListViewer");
        out.println("  dpi - delete the instances of template 'GroupWorkList'");
        out.println("  cpi - create some instances from template 'GroupWorkList'");
      }
      else
      {
        out.println("Logging on");

        Agent agent = new Agent();
        agent.setLocator(Agent.LOC_LOCATOR);
        agent.setName("MQWFAGENT");

        ExecutionService service = agent.locate("", "");
        service.logon2("ADMIN", "password",
                       SessionMode.PRESENT_HERE, AbsenceIndicator.RESET);

        if ("dwl".equals(args[0]))
        {
          WorkList[] worklists = service.queryWorkLists();

          for (int i = 0; i < worklists.length; ++i)
          {
            if (!"_WORKLIST".equals(worklists[i].name()) &&
                !"_GROUPLIST".equals(worklists[i].name()))
              out.println("Skipping work list " + worklists[i].name());
            else
            {
              out.println("Deleting work list " + worklists[i].name());
              worklists[i].delete();
            }
          }
        }
        else if ("dpi".equals(args[0]))
        {
          ProcessInstance[] instances =
             service.queryProcessInstances(null, null, null);

          for (int i = 0; i < instances.length; ++i)
          {
            if (!"GroupWorkList".equals(instances[i].processTemplateName()))
              out.println("Skipping instance " + instances[i].name());
            else
            {
              out.println("Deleting instance " + instances[i].name());
              instances[i].terminate();
            }
          }
        }
        else if ("cpi".equals(args[0]))
        {
          ProcessTemplate[] templates =
             service.queryProcessTemplates("NAME='GroupWorkList'", null, null);

          if (templates.length != 1)
            out.println("Template 'GroupWorkList' not found");
          else for (int i = 0; i < 20; ++i)
          {
            ReadWriteContainer inContainer = templates[0].initialInContainer();
            inContainer.setString("Data", "process #" + i);
            ProcessInstance instance =
               templates[0].createAndStartInstance2("GroupWorkList" + i,
                                                null, null, inContainer, false);
            out.println("Created instance " + instance.name());
          }
        }

        service.logoff();
        out.println("Done.");
      }
    }
    catch (Throwable xcpt)
    {
      xcpt.printStackTrace(out);
    }
  }
}
