<?xml version='1.0'?>
<!-- ================================================================================================================= -->
<!-- This style-sheet processes the XML-data generated by the AuditTrailHandler. It produces a tabular report, one row -->
<!-- for each 'AuditTrailRecord' element in the XML stream (in chronological order). It also demonstrates how a style- -->
<!-- sheet can utilize the Bean Scripting Framework (BSF) to call back to Java functions and to include JavaScript's   -->
<!-- in the transformation process.                                                                                    -->
<!-- ================================================================================================================= -->
<xsl:stylesheet
 xmlns:xsl     = 'http://www.w3.org/1999/XSL/Transform'
 xmlns:lxslt   = 'http://xml.apache.org/xslt'
 xmlns:java    = 'http://xml.apache.org/xslt/java'
 xmlns:js      = 'ext1'
 version       = '1.0'
 extension-element-prefixes='js'>
 <!-- ================================================================================================================ -->
 <!-- A JavaScript realizing an extension function. This function is used to transform milliseconds (since 01/01/1970  -->
 <!-- as delivered by the AuditTrailHandler) into a string representation. It also shows how a JavaScript can reach    -->
 <!-- back to Java. Here, a java.sql.Timestamp instance is used to perform the actual transformation by (implicitly)   -->
 <!-- calling its toString() method.                                                                                   -->
 <!-- ================================================================================================================ -->
 <lxslt:component prefix='js' functions='format'><lxslt:script lang='javascript'>

  var offset = java.util.TimeZone.getDefault().getRawOffset();

  function format(millis) { return new java.sql.Timestamp(parseInt(millis) + offset); }

 </lxslt:script></lxslt:component>
 <!-- ================================================================================================================ -->
 <!-- Formatting parameters for the Navigation bar. Those are passed by the AuditTrailHandler.                         -->
 <!-- ================================================================================================================ -->
 <xsl:param name='css'/>
 <xsl:param name='cmdBack'/>
 <xsl:param name='imgBack'/>
 <xsl:param name='cmdRefresh'/>
 <xsl:param name='imgRefresh'/>
 <xsl:param name='cmdLogoff'/>
 <xsl:param name='imgLogoff'/>
 <!-- ================================================================================================================ -->
 <!-- For the root element, produce the Navigation bar and recurse into the 'AuditTrail' tree.                         -->
 <!-- ================================================================================================================ -->
 <xsl:output method='html' indent='no'/>
 <xsl:template match='/'>
  <html>
   <head>
    <meta http-equiv='Content-Type' content='text/html; charset=iso-8859-1'/>
    <meta http-equiv='Expires' content='0'/>
    <meta http-equiv='Pragma' content='no-cache'/>
    <meta http-equiv='Cache-Control' content='no-cache'/>
    <title>IBM WebSphere MQ Workflow - Web Client - Audit Trail</title>
    <link>
     <xsl:attribute name='rel' >stylesheet</xsl:attribute>
     <xsl:attribute name='type'>text/css</xsl:attribute>
     <xsl:attribute name='href'><xsl:value-of select='$css'/></xsl:attribute>
    </link>
   </head>
   <body>
    <table width='100%' border='2' bordercolordark='#000000' bordercolorlight='#FFFFFF' cellpadding='0' cellspacing='0'
           class='navigator'>
     <tr>
      <td align='left'>
       <a>
        <xsl:attribute name='href'><xsl:value-of select='$cmdBack'/></xsl:attribute>
        <img>
         <xsl:attribute name='src'><xsl:value-of select='$imgBack'/></xsl:attribute>
         <xsl:attribute name='alt'>Back to Instance List</xsl:attribute>
         <xsl:attribute name='border'>0</xsl:attribute>
        </img>
       </a>
      </td>
      <td>
       <a>
        <xsl:attribute name='href'><xsl:value-of select='$cmdRefresh'/></xsl:attribute>
        <img>
         <xsl:attribute name='src'><xsl:value-of select='$imgRefresh'/></xsl:attribute>
         <xsl:attribute name='alt'>Refresh</xsl:attribute>
         <xsl:attribute name='border'>0</xsl:attribute>
        </img>
       </a>
      </td>
      <td width='100%'>&#160;</td>
      <td align='right'>
       <a>
        <xsl:attribute name='href'><xsl:value-of select='$cmdLogoff'/></xsl:attribute>
        <img>
         <xsl:attribute name='src'><xsl:value-of select='$imgLogoff'/></xsl:attribute>
         <xsl:attribute name='alt'>Log off</xsl:attribute>
         <xsl:attribute name='border'>0</xsl:attribute>
        </img>
       </a>
      </td>
     </tr>
    </table>
    <br/>
    <xsl:choose>
     <xsl:when test='count(AuditTrail/*) &gt; 0'>
      <xsl:apply-templates select='AuditTrail'/>
     </xsl:when>
     <xsl:otherwise>
      <h1>Audit Trail Data unavailable</h1>
      <p><b>No Audit Trail records for this process instance have been found. This means that no relevant audit events
      (as defined by the audit trail filter of this process) have occurred or this process is not audited at all.</b></p>
     </xsl:otherwise>
    </xsl:choose>
   </body>
  </html>
 </xsl:template>
 <!-- ================================================================================================================ -->
 <!-- Report common process data from first event, then generate a table, one row per audit event. The table can be    -->
 <!-- modified by adding/removing column headers and the corresponding xsl:value-of tag in the 'AuditTrailRecord'      -->
 <!-- template.                                                                                                        -->
 <!-- ================================================================================================================ -->
 <xsl:template match='AuditTrail'>
  <p>
   <font face='Arial' size='5'>
    <b>Audit Trail for Process Instance <i>&quot;<xsl:value-of select='AuditTrailRecord[1]/ProcInstName'/>&quot;</i></b>
   </font>
  </p>
  <table>
   <tr>
    <td><b>Template name</b></td>
    <td><b>:</b></td>
    <td><xsl:value-of select='AuditTrailRecord[1]/ProcTemplName'/></td>
   </tr>
   <tr>
    <td><b>Template valid since</b></td>
    <td><b>:</b></td>
    <td><xsl:value-of select='js:format(string(AuditTrailRecord[1]/ProcTemplValidFromDate))'/></td>
   </tr>
   <xsl:if test='AuditTrailRecord[1]/ProcInstParentName'>
    <tr>
     <td><b>Parent process name</b></td>
     <td><b>:</b>
     </td><td><xsl:value-of select='AuditTrailRecord[1]/ProcInstParentName'/></td>
    </tr>
   </xsl:if>
   <tr>
    <td><b>Toplevel process name</b></td>
    <td><b>:</b></td>
    <td><xsl:value-of select='AuditTrailRecord[1]/ProcInstTopLevelName'/></td>
   </tr>
  </table>
  <br/>
  <br/>
  <table border='3'>
   <tr bgcolor='#7B94E7'>
    <th>Creation Date</th>
    <th>Event</th>
    <th>Activity</th>
    <th>User</th>
    <th>User 2</th>
   </tr>
   <xsl:apply-templates select='AuditTrailRecord'>
    <xsl:sort select='Timestamp' order='ascending'/>
   </xsl:apply-templates>
  </table>
 </xsl:template>
 <!-- ================================================================================================================ -->
 <!-- Report details for each audit event. The time is converted to JDBC timestamp escape format (via the JavaScript   -->
 <!-- 'format' function). For absent optional elements a &#160; (non-breakable space) is generated.                    -->
 <!-- ================================================================================================================ -->
 <xsl:template match='AuditTrailRecord'>
  <tr bgcolor='yellow'>
   <td nowrap='true'>
    <xsl:value-of select='js:format(string(Timestamp))'/>
   </td>
   <td nowrap='true'>
    <xsl:apply-templates select='AuditEvent'/>
   </td>
   <td nowrap='true'>
    <xsl:if test='not(ActivityName)'>&#160;</xsl:if>
    <xsl:value-of select='ActivityName'/>
   </td>
   <td nowrap='true'>
    <xsl:if test='not(UserID)'>&#160;</xsl:if>
    <xsl:value-of select='UserID'/>
   </td>
   <td nowrap='true'>
    <xsl:if test='not(SecondUserID)'>&#160;</xsl:if>
    <xsl:value-of select='SecondUserID'/>
   </td>
  </tr>
 </xsl:template>
 <!-- ================================================================================================================ -->
 <!-- NLS enabled translation of Event codes, Activity types and States. This demonstrates the direct invocation of a  -->
 <!-- Java-based extension. Here, the passed code is replaced by an NLS-enabled, human-readable text string.           -->
 <!-- ================================================================================================================ -->
 <xsl:template match='AuditEvent | ActivityType | ActivityState'>
  <xsl:value-of select='java:com.ibm.workflow.servlet.sample.AuditTrailHandler.format(string(.))'/>
 </xsl:template>
</xsl:stylesheet>
