// *****************************************************************************
// * IBM WebSphere MQ Workflow Java API Beans - Example Application                *
// *                                                                           *
// * Licensed Materials - Property of IBM                                      *
// * 5621-290 5765-270 5622-615 5765-468 5697-216                              *
// * (C) Copyright IBM Corp. 1993, 2005                                        *
// * US Government Users Restricted Rights - Use, duplication or               *
// * disclosure restricted by GSA ADP Schedule Contract                        *
// * with IBM Corp.                                                            *
// *                                                                           *
// *****************************************************************************
// *                                                                           *
// * DISCLAIMER                                                                *
// *                                                                           *
// * This material contains programming source code for your consideration.    *
// * These examples have not been thoroughly tested under all conditions.      *
// * IBM, therefore, cannot guarantee or imply reliability, serviceability,    *
// * or function of these programs.                                            *
// * ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY    *
// * WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT      *
// * NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS       *
// * FOR A PARTICULAR PURPOSE.                                                 *
// *                                                                           *
// *****************************************************************************
// *                                                                           *
// * This application is the workflow equivalent of the typical                *
// * 'Hello World' example.                                                    *
// *                                                                           *
// *****************************************************************************

import com.ibm.workflow.api.*; // import the workflow API's
import com.ibm.workflow.api.ServicePackage.*;

public class HelloApplication
{

  public static void main(String[] args)
  {
    // Check the arguments. The first argument is the name of the IBM WebSphere MQ
    // Workflow agent the client will connect to. The second argument defines
    // the locator policy the client will use when trying to contact the agent.
    // The third/fourth argument define the userid/password, which, if not
    // specified, default to ADMIN and password
    //
    // Typically, invocations will have the following form:
    // LOC_LOCATOR - Local Bindings
    //   java HelloApplication AGENT LOC
    //

    if ((args.length < 2 ) || (args.length > 4 ))
    {
      System.out.println("Usage:");
      System.out.println(" java HelloApplication <agent_name> LOC [userid] [password]");
      System.exit(0);
    }
    try
    {
      // An agent bean representing an IBM WebSphere MQ Workflow domain
      String userid = "ADMIN";
      String passwd = "password";
      Agent agent = new Agent();
      agent.setContext(args, null);
      // Parse the command line and set the locator to be used to
      // communicate with the agent.
      if      (args[1].equalsIgnoreCase("LOC"))
      {
        agent.setLocator(Agent.LOC_LOCATOR);
      }
      else
      {
        System.out.println("Invalid locator policy: " + args[1]);
        System.exit(0);
      }
      if (args.length >=3 ) userid = args[2].toUpperCase();
      if (args.length >=4 ) passwd = args[3];

      // Set the name of the Agent to be contacted. Setting the name
      // automatically instructs the agent bean to contact the Agent using
      // the current locator policy. For this reason the 'setLocator' must be
      // called before 'setName' is invoked. If the agent bean cannot contact
      // the Agent, it will raise a java.beans.PropertyVetoException instead
      // of returning from the 'setName' call.
      agent.setName(args[0]);
      System.out.println("You are using " + agent.versionInfo().build);

      // Locate the default execution service in the default system group named
      // 'SYS_GRP' and the default system named 'FMCSYS' using the default
      // configuration. This call intentionally always returns successful
      // (to prevent intrusion attempts which guess at service names until they
      // find a valid one). Of course, only using a valid systemgroup and/or
      // system name will return an ExecutionService which can be used to log on.
      ExecutionService service = agent.locate("", "");

      // Log on to the execution service. If the UserID and/or the password is
      // invalid, a FmcException will be thrown.
      // do a normal logon
      // service.logon(userid, passwd);
      // do a forced logon
      service.logon2(userid, passwd, SessionMode.PRESENT_HERE,
                     AbsenceIndicator.LEAVE );
      System.out.println("Logon successful");

      // Query the set of worklists the logged on user can access.
      WorkList[] worklists = service.queryWorkLists();

      if (worklists.length == 0)
      {
        System.out.println("  No worklist found");
      }
      else
      {
        System.out.println("  available worklists: ");
        // Iterate over the worklists, printing out their names.
        for (int ndx = 0; ndx < worklists.length; ndx++)
        {
          System.out.println("     " + worklists[ndx].name());
        }

        // Query the set of workitems in the first worklist.
        System.out.println("  Workitems for worklist "+worklists[0].name());
        WorkItem[] workitems = worklists[0].queryWorkItems();

        // Iterate over the workitems, printing out their names.
        for (int ndx = 0; ndx < workitems.length; ndx++)
        {
          System.out.println("    " + workitems[ndx].name());
        }
      }/* End if*/

      // Query the available process templates.
      ProcessTemplate[] templates;
      templates = service.queryProcessTemplates("NAME LIKE *", null, null);

      if (templates.length == 0)
      {
        System.out.println("  No process templates found");
      }
      else
      {
        System.out.println("  available process templates: ");
        // Iterate over the templates, printing out their names.
        for (int ndx = 0; ndx < templates.length; ndx++)
        {
          System.out.println("     " + templates[ndx].name());
        }
      }/* End if*/

      // Logoff from the execution service. This (like any other remote call)
      // may raise an FmcException indicating a communication failure.
      service.logoff();

      System.out.println("Logoff successful");
    }
    catch(FmcException e)
    {
      // Catch and report details about the FmcException
      System.out.println("FmcException occured");
      System.out.println("  RC         : " + e.rc);
      System.out.println("  Origin     : " + e.origin);
      System.out.println("  MessageText: " + e.messageText);
      System.out.println("  Exception  : " + e.getMessage());
      System.out.println("  Parameters : ");
      for ( int i = 0; i < e.parameters.length ; i++)
      {
        System.out.println("    " + e.parameters[i] );
      }
      System.out.println("  StackTrace : ");
      e.printStackTrace();
    }
    catch(NoClassDefFoundError e)
    {
      System.out.println();
      System.out.println("A necessary class definition was not found. Please check your CLASSPATH");
      System.out.println("environment variable!");
      System.out.println("It should contain the file fmcjapi.jar instead of fmcojagt.jar.");
      System.out.println();
      System.out.println("Error message: ");
      e.printStackTrace();

    }

    catch(Exception e)
    {
      // Catch and report any exception that occurred.
      e.printStackTrace();
    }

    System.exit(0);
  }
}
